var gulp = require('gulp'),
	concat = require('gulp-concat'),
	del = require('del'),
	sourceMaps = require('gulp-sourcemaps'),
	stylus = require('gulp-stylus'),
	uglify = require('gulp-uglify'),
	minifyCss = require('gulp-minify-css'),
	gulpUtil = require('gulp-util'),
	bump = require('gulp-bump'),
	eslint = require('gulp-eslint'),
	jscs = require('gulp-jscs'),
	bowerConfig = require('./bower.json');

gulp.task('lint', ['eslint', 'jscs']);

gulp.task('build', ['clean'], function () {
	gulp.start(['js:development', 'stylus:development', 'js:release', 'stylus:release']);
});

gulp.task('release-with-patch-version-bump', ['clean', 'bump:patch'], function () {
	gulp.start(['js:release', 'stylus:release']);
});

gulp.task('release-with-minor-version-bump', ['clean', 'bump:minor'], function () {
	gulp.start(['js:release', 'stylus:release']);
});

gulp.task('release-with-major-version-bump', ['clean', 'bump:major'], function () {
	gulp.start(['js:release', 'stylus:release']);
});

gulp.task('watch', ['watch:js', 'watch:stylus']);

gulp.task('clean', function() {
	return del([
		'./dist/**/*'
	]);
});

gulp.task('js:development', function() {
	var jsSource = [
		'./src/ipconfigure.orchid.core.js',
		'./src/**/*.js'
	];

	return gulp.src(jsSource)
		.pipe(sourceMaps.init())
		.pipe(concat('ipconfigure.orchid.core.' + bowerConfig.version + '.js'))
		.pipe(sourceMaps.write())
		.pipe(gulp.dest('./dist/'));
});

gulp.task('stylus:development', function() {
	return gulp.src('./src/**/*.styl')
		.pipe(sourceMaps.init())
		.pipe(stylus())
		.pipe(concat('ipconfigure.orchid.core.' + bowerConfig.version + '.css'))
		.pipe(sourceMaps.write())
		.pipe(gulp.dest('./dist/'));
});

gulp.task('js:release', function() {
	var jsSource = [
		'./src/ipconfigure.orchid.core.js',
		'./src/**/*.js'
	];

	return gulp.src(jsSource)
			.pipe(sourceMaps.init())
			.pipe(concat('ipconfigure.orchid.core.' + bowerConfig.version + '.min.js'))
			.pipe(uglify())
			.pipe(sourceMaps.write())
			.pipe(gulp.dest('./dist/'));
});

gulp.task('stylus:release', function() {
	return gulp.src('./src/**/*.styl')
			.pipe(sourceMaps.init())
			.pipe(stylus())
			.pipe(concat('ipconfigure.orchid.core.' + bowerConfig.version + '.min.css'))
			.pipe(minifyCss())
			.pipe(sourceMaps.write())
			.pipe(gulp.dest('./dist/'));
});

gulp.task('beep', function() {
	gulpUtil.beep();
});

gulp.task('watch:js', function() {
	return gulp.watch('./src/**/*.js', ['js:development', 'beep']);
});

gulp.task('watch:stylus', function() {
	return gulp.watch('./src/**/*.styl', ['stylus:development', 'beep']);
});

gulp.task('bump:patch', function(){
	gulp.src('*.json')
			.pipe(bump({type:'patch'}))
			.pipe(gulp.dest('./'));
});

gulp.task('bump:minor', function(){
	gulp.src('*.json')
			.pipe(bump({type:'minor'}))
			.pipe(gulp.dest('./'));
});

gulp.task('bump:major', function(){
	gulp.src('*.json')
			.pipe(bump({type:'major'}))
			.pipe(gulp.dest('./'));
});

gulp.task('eslint', function () {
	return gulp.src(['src/**/*.js', 'tests/**/*.js'])
		.pipe(eslint())
		.pipe(eslint.format())
		.pipe(eslint.failAfterError());
});

gulp.task('jscs', function() {
	return gulp.src(['src/**/*.js', 'tests/**/*.js'])
		.pipe(jscs())
		.pipe(jscs.reporter());
});
