(function() {
	'use strict';

	angular.module('ipconfigure.orchid').controller('AdminCameraBulkDeleteCtrl', ['$scope', '$filter', '$http', '$log', AdminCameraBulkDeleteCtrl]);

	function AdminCameraBulkDeleteCtrl($scope, $filter, $http, $log) {
		function _deleteCamera(baseUrl, camera) {
			return $http.delete(baseUrl + '/service/cameras/' + camera.id);
		}

		$scope.selectCamera = function(camera) {
			if ($scope.processing !== true) {
				camera.markedForDeletion = !camera.markedForDeletion;

				$scope.ready = $scope.configuredCameras.some(function(configuredCamera) { return configuredCamera.markedForDeletion; });
			}
		};

		$scope.showError = function(camera) {
			alert(camera.error);
		};

		$scope.deleteCameras = function() {
			var index = 0;

			if (confirm('Are you sure?') === true) {
				$scope.processing = true;
				$scope.ready = false;

				// sort cameras so we can register them in order according to the UI
				$scope.configuredCameras = $filter('orderBy')($scope.configuredCameras, 'name');

				(function go() {
					if ($scope.configuredCameras.length > index) {
						delete $scope.configuredCameras[index].error;

						if ($scope.configuredCameras[index].markedForDeletion === true) {
							$scope.configuredCameras[index].markedForDeletionProcessing = true;

							_deleteCamera($scope.configuredCameras[index].instance.baseUrl, $scope.configuredCameras[index])
								.success(function() {
									// remove the camera from the parent controller (AdminCtrl)
									$scope.$parent.configuredCameras.forEach(function(camera, parentIndex) {
										if (camera === $scope.configuredCameras[index]) {
											$scope.$parent.configuredCameras.splice(parentIndex, 1);
										}
									});

									// remove the camera from our local copy of configuredCameras
									$scope.configuredCameras.splice(index, 1);

									if ($scope.destroyed === false) {
										go();
									}
								})
								.error(function(data, status, headers, config) {
									$scope.configuredCameras[index].markedForDeletionProcessing = false;

									if (data && (data.code || data.reason)) {
										$scope.configuredCameras[index].error = data.code + ': ' + data.reason;
									}
									else {
										$scope.configuredCameras[index].error = 'An unexpected error occurred.';
									}

									index += 1;

									if (status !== 0) {
										$log.error('error deleting camera #' + $scope.configuredCameras[index], data, status, config);
									}

									if ($scope.destroyed === false) {
										go();
									}
								});
						}
						else {
							index += 1;

							if ($scope.destroyed === false) {
								go();
							}
						}
					}
					else {
						// we're done
						$scope.processing = false;
						$scope.ready = $scope.configuredCameras.some(function(camera) { return camera.markedForDeletion; });
					}
				}());
			}
		};

		$scope.$on('$destroy', function() {
			// cancel the delete loop
			$scope.destroyed = true;
		});

		// init
		$scope.destroyed = false;
		$scope.processing = false;
		$scope.ready = false;
	}
}());
