(function() {
	'use strict';

	angular.module('ipconfigure.orchid').controller('AdminCameraDiscoverCtrl', ['$scope', '$http', '$log', '$timeout', 'FocusOnService', 'OrchidDiscoveryService', 'ScrollToOnService', AdminCameraDiscoverCtrl]);

	function AdminCameraDiscoverCtrl($scope, $http, $log, $timeout, FocusOnService, OrchidDiscoveryService, ScrollToOnService) {
		$scope.cameraRegistration = {
			instance: OrchidDiscoveryService.currentInstance
		};
		$scope.failure = '';
		$scope.import = false;
		$scope.processing = false;
		$scope.orchidInstances = OrchidDiscoveryService.instances;

		$scope.$watch('selectedDiscoveredCamera', function() {
			$scope.cameraRegistration = {
				import: false
			};

			$scope.failure = '';

			$scope.cameraRegistration.instance = OrchidDiscoveryService.currentInstance;

			ScrollToOnService('top');

			// allow for scroll to happen
			$timeout(function() {
				FocusOnService('initial-field');
			}, 275);
		});

		$scope.registerCamera = function() {
			$scope.processing = true;

			$http.post($scope.cameraRegistration.instance.baseUrl + '/service/cameras', {
				driver: 'ONVIF',
				name: $scope.cameraRegistration.name,
				import: $scope.cameraRegistration.import,
				connection: {
					uri: $scope.selectedDiscoveredCamera.url,
					username: $scope.cameraRegistration.username,
					password: $scope.cameraRegistration.password,
					ip: $scope.selectedDiscoveredCamera.ip
				}
			})
				.success(function(response) {
					$log.debug('registration complete', response);

					response.instance = $scope.cameraRegistration.instance;

					// not entirely sure about this -- we're essentially modifying the parent scope
					$scope.configuredCameras.push(response);
					$scope.goCameraEdit(response);

					$scope.failure = '';
					$scope.processing = false;
				})
				.error(function(data, status, headers, config) {
					$scope.processing = false;

					if (data && (data.code || data.reason)) {
						$scope.failure = data.code + ': ' + data.reason;
					}
					else {
						$scope.failure = 'An unexpected error occurred.';
					}

					if (status !== 0) {
						$log.error('error, POST /service/cameras', data, status, config);
					}
				});
		};
	}
}());
