(function() {
	'use strict';

	angular.module('ipconfigure.orchid').controller('AdminCameraEditCtrl', ['$scope', '$filter', '$http', '$log', '$timeout', 'FocusOnService', 'RecordingStyleService', 'ScrollToOnService', 'StreamService', AdminCameraEditCtrl]);

	function AdminCameraEditCtrl($scope, $filter, $http, $log, $timeout, FocusOnService, RecordingStyleService, ScrollToOnService, StreamService) {
		$scope.processingCameraInformation = false;
		$scope.processingCameraSettings = false;
		$scope.processingPrimaryStream = false;
		$scope.processingStream = false;
		$scope.processingDeleteStream = {};

		$scope.failureCameraInformation = '';
		$scope.failureCameraSettings = '';
		$scope.failureStream = '';

		$scope.refreshingPrimaryStreamStatus = false;
		$scope.streamState = 'unknown';
		$scope.retryCount = 0;

		$scope.recordingStyles = RecordingStyleService.getAllRecordingStyles();

		$scope.refreshPrimaryStreamStatus = function() {
			if ($scope.selectedConfiguredCamera.primaryStream && $scope.selectedConfiguredCamera.primaryStream.id) {
				$scope.refreshingPrimaryStreamStatus = true;

				$timeout(function() {
					$http.get($scope.selectedConfiguredCamera.instance.baseUrl + '/service/streams/' + $scope.selectedConfiguredCamera.primaryStream.id + '/status')
						.success(function(status) {
							$scope.streamState = status.streamState;
							$scope.retryCount = status.retryCount || 0;
							$scope.refreshingPrimaryStreamStatus = false;
						})
						.error(function(data, status, headers, config) {
							$scope.streamState = 'unable to retrieve';
							$scope.retryCount = 0;
							$scope.refreshingPrimaryStreamStatus = false;

							if (status !== 0) {
								$log.error('error, GET /service/streams/' + $scope.selectedConfiguredCamera.primaryStream.id + '/status', data, status, headers, config);
							}
						});
				}, 250);
			}
		};

		$scope.updateCameraName = function(shadow, shadowCommitFn, form) {
			$scope.processingCameraInformation = true;

			$http.patch(shadow.instance.baseUrl + '/service/cameras/' + shadow.id, { name: shadow.name })
				.success(function() {
					shadowCommitFn({ ignore: ['active', 'capabilities', 'configuration', 'driver', 'href', 'id', 'primaryStream', 'retention', 'server', 'streams', 'instance'] });
					StreamService.forceUpdate(); // sucks

					form.$setPristine();

					FocusOnService('camera-name');

					$scope.processingCameraInformation = false;
					$scope.failureCameraInformation = '';
				})
				.error(function(data, status, headers, config) {
					$scope.processingCameraInformation = false;

					if (data && (data.code || data.reason)) {
						$scope.failureCameraInformation = data.code + ': ' + data.reason;
					}
					else {
						$scope.failureCameraInformation = 'An unexpected error occurred.';
					}

					if (status !== 0) {
						$log.error('error, PATCH /service/cameras/' + shadow.id, data, status, headers, config);
					}
				});
		};

		$scope.updateCameraSettings = function(shadow, shadowCommitFn, form) {
			$scope.processingCameraSettings = true;

			$http.patch(shadow.instance.baseUrl + '/service/cameras/' + shadow.id, { configuration: shadow.configuration })
				.success(function(response) {
					var key;

					// update all properties on $scope.selectedStream
					for (key in response.camera) {
						if (response.camera.hasOwnProperty(key) === true) {
							$log.debug('set prop', key, response.camera[key]);
							shadow[key] = response.camera[key];
						}
					}

					_createObjectArray(shadow.capabilities, 'camera');

					// TODO handle diff

					shadowCommitFn();
					StreamService.forceUpdate(); // sucks

					form.$setPristine();

					$scope.processingCameraSettings = false;
					$scope.failureCameraSettings = '';
				})
				.error(function(data, status, headers, config) {
					$scope.processingCameraSettings = false;

					if (data && (data.code || data.reason)) {
						$scope.failureCameraSettings = data.code + ': ' + data.reason;
					}
					else {
						$scope.failureCameraSettings = 'An unexpected error occurred.';
					}

					if (status !== 0) {
						$log.error('error, PATCH /service/cameras/' + shadow.id, data, status, headers, config);
					}
				});
		};

		$scope.deleteCamera = function(camera) {
			if (window.confirm('Are you sure?') === true) {
				$scope.deleting = true;

				$http.delete(camera.instance.baseUrl + '/service/cameras/' + camera.id)
					.success(function() {
						$scope.$emit('change-admin-include', 'admin-camera-delete-confirm');

						// find camera in array and splice it, not entirely sure about this -- we're essentially modifying the parent scope
						$scope.configuredCameras.some(function(e, i) {
							if (e.id === camera.id) {
								$scope.configuredCameras.splice(i, 1);
								return true;
							}
						});

						delete $scope.selectedConfiguredCamera;

						$scope.deleting = false;
					})
					.error(function(data, status, headers, config) {
						$scope.deleting = false;

						if (status !== 0) {
							$log.error(data, status, headers, config);
							alert('cameras/' + camera.id + ' delete failed');
						}
					});
			}
		};

		$scope.recoverCamera = function(connection) {
			$scope.processing = true;
			$scope.processingRecover = true;

			$http.patch($scope.selectedConfiguredCamera.instance.baseUrl + '/service/cameras/' + $scope.selectedConfiguredCamera.id, { connection: connection })
				.success(function(response) {
					connection.password = '';
					connection.username = '';
					connection.uri = '';

					// overwrite the camera from the server, preserving the instance
					response.camera.instance = $scope.selectedConfiguredCamera.instance;
					angular.copy(response.camera, $scope.selectedConfiguredCamera);

					// reinitialize the selected camera
					_initializeSelectedConfiguredCamera();

					$scope.processing = false;
					$scope.processingRecover = false;
					$scope.failureRecover = '';
					$scope.recovering = false;
				})
				.error(function(data) {
					$scope.processing = false;
					$scope.processingRecover = false;

					if (data && (data.code || data.reason)) {
						$scope.failureRecover = data.code + ': ' + data.reason;
					}
					else {
						$scope.failureRecover = 'An unexpected error occurred.';
					}
				});
		};

		$scope.prepareAddStream = function() {
			var streamCount = Array.isArray($scope.selectedConfiguredCamera.streams) === true ? $scope.selectedConfiguredCamera.streams.length : 0;

			if (parseInt($scope.selectedConfiguredCamera.capabilities.camera.PossibleStreams, 10) <= streamCount) {
				alert('This camera has reached the maximum number of streams allowed.  Please delete or modify an existing stream.');
			}
			else {
				$scope.selectStream($scope.createNewStream(), $scope.streamConfigurationForm);
			}
		};

		$scope.createNewStream = function() {
			var category,
				newStream = { configuration: {}, recordingConfiguration: {} };

			// set configuration defaults
			for (category in $scope.selectedConfiguredCamera.capabilities.stream) {
				if ($scope.selectedConfiguredCamera.capabilities.stream.hasOwnProperty(category) === true) {
					newStream.configuration[category] = {};
					_setStreamDefaults(newStream.configuration[category], $scope.selectedConfiguredCamera.capabilities.stream[category]);
				}
			}

			// set recordingConfiguration defaults
			newStream.recordingConfiguration.bgseg = {};
			for (category in $scope.selectedConfiguredCamera.instance.recordingCapabilities.bgseg) {
				if ($scope.selectedConfiguredCamera.instance.recordingCapabilities.bgseg.hasOwnProperty(category) === true) {
					newStream.recordingConfiguration.bgseg[category] = $scope.selectedConfiguredCamera.instance.recordingCapabilities.bgseg[category].default;
				}
			}

			newStream.recordingConfiguration.filesplit = {};
			for (category in $scope.selectedConfiguredCamera.instance.recordingCapabilities.filesplit) {
				if ($scope.selectedConfiguredCamera.instance.recordingCapabilities.filesplit.hasOwnProperty(category) === true) {
					newStream.recordingConfiguration.filesplit[category] = $scope.selectedConfiguredCamera.instance.recordingCapabilities.filesplit[category].default;
				}
			}

			newStream.recordingConfiguration.mode = $scope.selectedConfiguredCamera.instance.recordingCapabilities.mode.default;
			newStream.recordingConfiguration.motionReducer = $scope.selectedConfiguredCamera.instance.recordingCapabilities.motionReducer.default;

			// set default recording style
			newStream.recordingStyle = RecordingStyleService.getDefaultRecordingStyle().name;

			return newStream;
		};

		$scope.selectStream = function(stream) {
			var category;

			if ($scope.processingDeleteStream[stream.id] !== true) {
				$scope.failureStream = '';
				$scope.selectedStream = stream;
				$scope.streamConfigurationForm.$setPristine();

				FocusOnService('initial-stream-field');

				for (category in $scope.selectedConfiguredCamera.capabilities.stream) {
					if ($scope.selectedConfiguredCamera.capabilities.stream.hasOwnProperty(category) === true) {
						_createObjectArray($scope.selectedConfiguredCamera.capabilities.stream, category);
					}
				}

				$timeout(function() {
					ScrollToOnService('stream-selected');
				}, 100);
			}
		};

		$scope.addStream = function(stream, form) {
			$scope.processingStream = true;

			_determineRecordingConfigurationForRecordingStyle(stream);

			$http.post($scope.selectedConfiguredCamera.instance.baseUrl + '/service/cameras/' + $scope.selectedConfiguredCamera.id + '/streams', stream)
				.success(function(response) {
					if (Array.isArray($scope.selectedConfiguredCamera.streams) === false) {
						$scope.selectedConfiguredCamera.streams = [];
					}

					// make necessary modifications (should be using a meta object, sorry)
					response.stream.recordingStyle = RecordingStyleService.determineRecordingStyleFromStream(response.stream).name;

					// set stream as primary if it's the only one
					if ($scope.selectedConfiguredCamera.streams.length === 0) {
						if (angular.isUndefined($scope.selectedConfiguredCamera.primaryStream) === true) {
							$scope.selectedConfiguredCamera.primaryStream = {};
						}

						angular.copy({ href: response.stream.href, id: response.stream.id }, $scope.selectedConfiguredCamera.primaryStream);

						_setSelectedConfiguredCameraPrimaryStreamRtspUrl(response.stream);
					}

					// add the stream to the camera
					$scope.selectedConfiguredCamera.streams.push(response.stream);

					// after we've set everything up, if this is the only camera, tell the StreamService to update
					if ($scope.selectedConfiguredCamera.streams.length === 1) {
						StreamService.forceUpdate();
					}

					if (ipConfigure.core.isEmpty(response.diff) === true) {
						// make it so that a stream is no longer selected
						$scope.selectedStream = undefined;
					}
					else {
						// TODO handle diff by highlighting selected fields and showing a warning instead of an error
						//						for (key in response.diff) {
						//							if (response.diff.hasOwnProperty(key)) {
						//								return false;
						//							}
						//						}

						$scope.selectedStream = response.stream;

						$timeout(function() {
							alert('Stream was updated, but some settings could not be accepted as-is.');
						});
					}

					form.$setPristine();

					$scope.processingStream = false;
					$scope.failureStream = '';
				})
				.error(function(data, status, headers, config) {
					$scope.processingStream = false;

					if (data && (data.code || data.reason)) {
						$scope.failureStream = data.code + ': ' + data.reason;
					}
					else {
						$scope.failureStream = 'An unexpected error occurred.';
					}

					if (status !== 0) {
						$log.error('error, PUT /service/cameras/' + $scope.selectedConfiguredCamera.id + '/streams', data, status, headers, config);
					}
				});
		};

		$scope.updateStream = function(camera, shadowStream, shadowCommitFn, form) {
			var copiedStream = angular.copy(shadowStream);

			// this is to avoid the circular reference when serializing
			delete copiedStream.cameraExpanded;

			$log.debug('update-stream', camera, shadowStream);

			_determineRecordingConfigurationForRecordingStyle(copiedStream);

			$scope.processingStream = true;

			$http.put(camera.instance.baseUrl + '/service/cameras/' + camera.id + '/streams/' + shadowStream.id, copiedStream)
				.success(function(response) {
					// update all properties on $scope.selectedStream
					angular.copy(response.stream, shadowStream);

					// make necessary modifications (should be using a meta object, sorry)
					shadowStream.recordingStyle = RecordingStyleService.determineRecordingStyleFromStream(shadowStream).name;

					if (ipConfigure.core.isEmpty(response.diff) === true) {
						// make it so that a stream is no longer selected
						$scope.selectedStream = undefined;
					}
					else {
						// TODO handle diff by highlighting selected fields and showing a warning instead of an error
						//						for (key in response.diff) {
						//							if (response.diff.hasOwnProperty(key)) {
						//								return false;
						//							}
						//						}

						$timeout(function() {
							alert('Stream was updated, but some settings could not be accepted as-is.');
						});
					}

					form.$setPristine();
					shadowCommitFn();
					StreamService.forceUpdate();

					$scope.processingStream = false;
				})
				.error(function(data, status, headers, config) {
					$scope.processingStream = false;

					if (data && (data.code || data.reason)) {
						$scope.failureStream = data.code + ': ' + data.reason;
					}
					else {
						$scope.failureStream = 'An unexpected error occurred.';
					}

					if (status !== 0) {
						$log.debug('error, PUT /service/cameras/' + camera.id + '/streams/' + shadowStream.id, data, status, headers, config);
					}
				});
		};

		$scope.makeStreamPrimary = function(camera, stream, $event) {
			var oldPrimaryStreamId;

			$event.stopPropagation();

			oldPrimaryStreamId = camera.primaryStream.id;

			$scope.processingPrimaryStream = true;
			camera.primaryStream.id = stream.id;

			$http.patch(camera.instance.baseUrl + '/service/cameras/' + camera.id, {
					primaryStream: {
						id: stream.id
					}
				})
				.success(function() {
					$scope.processingPrimaryStream = false;

					StreamService.forceUpdate();
				})
				.error(function(data, status, headers, config) {
					$scope.processingPrimaryStream = false;

					// rollback
					camera.primaryStream.id = oldPrimaryStreamId;

					if (status !== 0) {
						$log.debug('error, PATCH /service/cameras/' + camera.id, data, status, headers, config);

						alert('Unable to mark stream as primary.');
					}
				});
		};

		$scope.deleteStream = function(camera, stream, $event) {
			$event.stopPropagation();

			if (window.confirm('Are you sure?') === true) {
				$scope.processingDeleteStream[stream.id] = true;

				$http.delete(camera.instance.baseUrl + '/service/cameras/' + camera.id + '/streams/' + stream.id)
					.success(function(response) {
						if ($scope.selectedStream && $scope.selectedStream.id === stream.id) {
							$scope.selectedStream = undefined;
						}

						// find stream in array and splice it
						$scope.configuredCameras.some(function(configuredCamera) {
							if (configuredCamera.id === camera.id) {
								configuredCamera.streams.some(function(configuredStream, configuredStreamIndex) {
									if (configuredStream.id === stream.id) {
										configuredCamera.streams.splice(configuredStreamIndex, 1);
										return true;
									}
								});
								return true;
							}
						});

						// update primaryStream
						angular.copy(response.primaryStream, $scope.selectedConfiguredCamera.primaryStream);

						// set the rtsp url on the new primary stream
						camera.streams.some(function(cameraStream) {
							if (cameraStream.id === camera.primaryStream.id) {
								_setSelectedConfiguredCameraPrimaryStreamRtspUrl(cameraStream);
								return true;
							}
						});

						StreamService.forceUpdate();

						$scope.processingDeleteStream[stream.id] = false;
					})
					.error(function(data, status, headers, config) {
						$scope.processingDeleteStream[stream.id] = false;

						if (status !== 0) {
							$log.error(data, status, headers, config);
							alert('cameras/' + camera.id + '/streams/' + stream.id + ' delete failed');
						}
					});
			}
		};

		$scope.initRecoverCamera = function() {
			$scope.recovering = true;
			$scope.recoveringForced = false;

			$timeout(function() {
				FocusOnService('initial-recovery-field');
				ScrollToOnService('recover-camera');
			});
		};

		$scope.cancelRecoverCamera = function() {
			$scope.recovering = false;
		};

		$scope.$watch('selectedConfiguredCamera', function() {
			_initializeSelectedConfiguredCamera();
			FocusOnService('initial-field');
		});

		function _createObjectArray(parentObj, newKey) {
			var i,
				key;

			if (!parentObj.categoryArrays) {
				parentObj.categoryArrays = {};
			}

			// if the array property hasn't been created yet and there is a source object available (audio caps won't be set for cameras without audio capabilities)
			if (!parentObj.categoryArrays[newKey + 'Array'] && parentObj[newKey]) {
				// convert the objects into arrays so that we can display the controls in their natural order...
				// if you try to iterate through key/values in ngRepeat, angular will sort the keys.
				parentObj.categoryArrays[newKey + 'Array'] = $filter('objectToArray')(parentObj[newKey], true);

				// filter out non-typed objects, because we'll have no idea how to display them (PTZ for example)
				for (i = 0; i < parentObj.categoryArrays[newKey + 'Array'].length; i += 1) {
					if (parentObj.categoryArrays[newKey + 'Array'][i].value && typeof parentObj.categoryArrays[newKey + 'Array'][i].value.type !== 'string') {
						parentObj.categoryArrays[newKey + 'Array'].splice(i, 1);
						i -= 1;
					}
				}

				for (i = 0; i < parentObj.categoryArrays[newKey + 'Array'].length; i += 1) {
					if (parentObj.categoryArrays[newKey + 'Array'][i].value.capabilities) {
						for (key in parentObj.categoryArrays[newKey + 'Array'][i].value.capabilities) {
							if (parentObj.categoryArrays[newKey + 'Array'][i].value.capabilities.hasOwnProperty(key)) {
								if (!parentObj.categoryArrays[newKey + 'Array'][i].value.capabilitiesArray) {
									parentObj.categoryArrays[newKey + 'Array'][i].value.capabilitiesArray = {};
								}

								parentObj.categoryArrays[newKey + 'Array'][i].value.capabilitiesArray[key] = $filter('objectToArray')(parentObj.categoryArrays[newKey + 'Array'][i].value.capabilities[key]);
							}
						}
					}
				}

				$log.debug('created ' + newKey + 'Array', parentObj.categoryArrays[newKey + 'Array']);
			}

		}

		function _determineRecordingConfigurationForRecordingStyle(stream) {
			var success = RecordingStyleService.setRecordingConfigurationForStreamFromRecordingStyle(stream);

			if (success === false) {
				alert('Unknown recording style chosen ' + stream.recordingStyle);
			}
		}

		function _initializeSelectedConfiguredCamera() {
			$scope.recovering = false;
			$scope.recoveringForced = true;
			$scope.selectedStream = undefined;
			//$scope.cameraSettingsForm.$setPristine();
			$scope.failureCameraSettings = '';
			$scope.failureStream = '';
			$scope.connection = { uri: 'http://' + $scope.selectedConfiguredCamera.configuration.IPAddress + '/onvif/device_service' };

			if (Array.isArray($scope.selectedConfiguredCamera.streams) === true) {
				$scope.selectedConfiguredCamera.streams.forEach(function(stream) {
					if (stream.id === $scope.selectedConfiguredCamera.primaryStream.id) {
						_setSelectedConfiguredCameraPrimaryStreamRtspUrl(stream);
					}

					stream.recordingStyle = RecordingStyleService.determineRecordingStyleFromStream(stream).name;
				});

				$scope.refreshPrimaryStreamStatus();
			}

			_createObjectArray($scope.selectedConfiguredCamera.capabilities, 'camera');
		}

		function _setStreamDefaults(streamConfiguration, capabilities) {
			var key;

			for (key in capabilities) {
				if (capabilities.hasOwnProperty(key) === true) {
					streamConfiguration[key] = capabilities[key].default;

					if (capabilities[key].capabilities && capabilities[key].capabilities[streamConfiguration[key]]) {
						_setStreamDefaults(streamConfiguration, capabilities[key].capabilities[streamConfiguration[key]]);
					}
				}
			}
		}

		function _setSelectedConfiguredCameraPrimaryStreamRtspUrl(stream) {
			if ($scope.selectedConfiguredCamera.driver === 'ONVIF') {
				$scope.selectedConfiguredCamera.primaryStream.rtsp = stream.configuration.Resource;
			}
			else {
				$scope.selectedConfiguredCamera.primaryStream.rtsp = $scope.selectedConfiguredCamera.configuration.BaseUri + stream.configuration.General.Path;
			}
		}
	}
}());
