(function() {
	'use strict';

	angular.module('ipconfigure.orchid').controller('AdminScheduleCtrl', ['$scope', '$http', '$log', 'FocusOnService', 'OrchidDiscoveryService', AdminScheduleCtrl]);

	function AdminScheduleCtrl($scope, $http, $log, FocusOnService, OrchidDiscoveryService) {
		$scope.processingDelete = {};
		$scope.loading = false;
		$scope.schedules = [
			{
				id: 1,
				name: 'Example Schedule',
				associatedStreams: [1, 2, 3, 4],
				recordingSegments: [
					{ type: 'recurDayOfWeek', typeValues: '0', startDate: 1404208800000, duration: 420 },
					{ type: 'recurDayOfWeek', typeValues: '0', startDate: 1404238500000, duration: 255 },
					{ type: 'recurDayOfWeek', typeValues: '1', startDate: 1404187200000, duration: 660 },
					{ type: 'recurDayOfWeek', typeValues: '1', startDate: 1404234000000, duration: 300 },
					{ type: 'recurDayOfWeek', typeValues: '2', startDate: 1404187200000, duration: 1440 }
				]
			},
			{
				id: 2,
				name: 'Weekends Only',
				associatedStreams: [],
				recordingSegments: [
					{ type: 'recurDayOfWeek', typeValues: '0', startDate: 1404187200000, duration: 1440 },
					{ type: 'recurDayOfWeek', typeValues: '6', startDate: 1404187200000, duration: 1440 }
				]
			},
			{
				id: 3,
				name: 'Nighttime Hours',
				associatedStreams: [5],
				recordingSegments: [
					{ type: 'recurDayOfWeek', typeValues: '0', startDate: 1404208800000, duration: 420 },
					{ type: 'recurDayOfWeek', typeValues: '0', startDate: 1404238500000, duration: 255 },
					{ type: 'recurDayOfWeek', typeValues: '1', startDate: 1404187200000, duration: 660 },
					{ type: 'recurDayOfWeek', typeValues: '1', startDate: 1404234000000, duration: 300 },
					{ type: 'recurDayOfWeek', typeValues: '2', startDate: 1404187200000, duration: 1440 }
				]
			}
		];

		$scope.createNewSchedule = function() {
			$scope.selectedSchedule = {
				streamCount: 0
			};
		};

		$scope.selectSchedule = function(schedule, scheduleForm) {
			$scope.selectedSchedule = schedule;

			// normalize data
			$scope.selectedSchedule.normalizedRecordingSegments = { 0: [], 1: [], 2: [], 3: [], 4: [], 5: [], 6: [] };

			$scope.selectedSchedule.recordingSegments.forEach(function(segment) {
				var dayOfWeek,
					normalizedSegment,
					startDate,
					startInMinutes;

				// currently we're only supporting this type
				if (segment.type === 'recurDayOfWeek') {
					dayOfWeek = JSON.parse(segment.typeValues);

					if (typeof dayOfWeek === 'number') {
						// initialize the segment if it hasn't already been
						if (angular.isUndefined($scope.selectedSchedule.normalizedRecordingSegments[dayOfWeek]) === true) {
							$scope.selectedSchedule.normalizedRecordingSegments[dayOfWeek] = [];
						}

						normalizedSegment = $scope.selectedSchedule.normalizedRecordingSegments[dayOfWeek];
						startDate = new Date(segment.startDate);
						startInMinutes = startDate.getHours() * 60 + startDate.getMinutes();

						normalizedSegment.push({
							start: startInMinutes,
							duration: segment.duration
						});
					}
				}
			});

			if (scheduleForm) {
				scheduleForm.$setPristine();
			}

			FocusOnService('initial-field');
		};

		$scope.updateSchedule = function(shadowSchedule, shadowCommitFn) {
			var recordingSegments = [],
				weekday,
				i,
				normalizedSegment,
				startDate,
				hours,
				minutes;

			// denormalize the schedule by looping through each weekday
			for (weekday in shadowSchedule.normalizedRecordingSegments) {
				if (shadowSchedule.normalizedRecordingSegments.hasOwnProperty(weekday) === true) {
					// now loop through each normalized segment for that weekday
					for (i = 0; i < shadowSchedule.normalizedRecordingSegments[weekday].length; i += 1) {
						// denormalize the segment by converting start/duration back to the recording schedule schema
						normalizedSegment = shadowSchedule.normalizedRecordingSegments[weekday][i];
						startDate = new Date();
						hours = Math.floor(normalizedSegment.start / 60);
						minutes = (normalizedSegment.start % 60);

						startDate.setHours(hours);
						startDate.setMinutes(minutes);
						startDate.setSeconds(0);
						startDate.setMilliseconds(0);

						recordingSegments.push({
							type: 'recurDayOfWeek',
							typeValues: weekday.toString(),
							startDate: startDate.getTime(),
							duration: normalizedSegment.duration
						});
					}
				}
			}

			// update the schedule in memory
			$scope.selectedSchedule = undefined;
			shadowSchedule.recordingSegments = recordingSegments;
			shadowCommitFn();

			// TODO persist to the web service
		};

		$scope.deleteSchedule = function(schedule, $event) {
			$event.stopPropagation();

			if (window.confirm('Are you sure?') === true) {
				$scope.processingDelete[schedule.id] = true;

				OrchidDiscoveryService.fetchCurrentServiceInstance().then(function(serviceInstance) {
					$http.delete(serviceInstance.baseUrl + '/service/schedule/' + schedule.id)
						.success(function() {
							if ($scope.selectedSchedule && $scope.selectedSchedule.id === schedule.id) {
								$scope.selectedSchedule = undefined;
							}

							// find stream in array and splice it
							$scope.schedules.some(function(configuredSchedule, configuredScheduleIndex) {
								if (configuredSchedule.id === schedule.id) {
									$scope.schedules.splice(configuredScheduleIndex, 1);
									return true;
								}
							});

							$scope.processingDelete[schedule.id] = false;
						})
						.error(function(data, status, headers, config) {
							$scope.processingDelete[schedule.id] = false;

							if (status !== 0) {
								$log.error(data, status, headers, config);
							}
						});
				});
			}
		};
	}
}());
