(function() {
	'use strict';

	angular.module('ipconfigure.orchid').controller('AdminSystemReportDailyCtrl', ['$scope', '$http', '$log', 'BaseServiceUrl', AdminSystemReportDailyCtrl]);

	function AdminSystemReportDailyCtrl($scope, $http, $log, BaseServiceUrl) {
		var fetching = false;

		$scope.from = new Date();
		$scope.to = new Date();
		$scope.from.setHours(0, 0, 0, 0);
		$scope.to.setHours(23, 59, 0, 0);

		$scope.streams = [];
		$scope.events = [];
		$scope.selectedEvents = [];

		$scope.pointClick = function(events) {
			$scope.selectedEvents = events;

			// lazy load descriptive where
			$scope.selectedEvents.forEach(function(event) {
				if (event.hasOwnProperty('descriptiveWhere') === false) {
					event.descriptiveWhere = $scope.lookupWhere(event.where);
				}
			});
		};

		$scope.lookupWhere = function(where) {
			var filtered;

			if (where === 'orchid') {
				return where;
			}
			else {
				// lookup stream (would be quicker if we had a hash table)
				filtered = $scope.streams.filter(function(stream) {
					if (stream.streamId === where) {
						return true;
					}
				});

				if (filtered.length === 0) {
					return 'unknown camera/stream';
				}
				else {
					return filtered[0].cameraName + ' / ' + filtered[0].name;
				}
			}
		};

		function fetchReport() {
			if (fetching === false && angular.isDefined($scope.from) === true && angular.isDefined($scope.to) === true) {
				fetching = true;

				$http.get(BaseServiceUrl + '/service/server/report?start=' + $scope.from.getTime() + '&stop=' + $scope.to.getTime())
					.success(function(response) {
						fetching = false;

						$scope.streams = [];
						$scope.events = [];
						$scope.selectedEvents = [];

						// flatten the list of cameras+streams
						if (response.cameras) {
							response.cameras.forEach(function(camera) {
								camera.streams.forEach(function(stream) {
									var copiedStream = angular.copy(stream);
									copiedStream.cameraName = camera.name;
									$scope.streams.push(copiedStream);
								});
							});
						}

						// combine orchid and stream events
						$scope.events = [];

						if (response.orchidEvents) {
							response.orchidEvents.forEach(function(orchidEvent) {
								$scope.events.push({
									what: orchidEvent.type,
									where: 'orchid',
									descriptiveWhere: 'orchid',
									when: orchidEvent.when
								});
							});
						}

						if (response.streamEvents) {
							response.streamEvents.forEach(function(streamEvent) {
								// descriptiveWhere to be lazy-loaded when needed
								$scope.events.push({
									what: 'stream-' + streamEvent.type + (streamEvent.message ? ' - ' + streamEvent.message : ''),
									where: streamEvent.streamId,
									when: streamEvent.when
								});
							});
						}
					})
					.error(function(data, status, headers, config) {
						fetching = false;

						$scope.streams = [];
						$scope.events = [];
						$scope.selectedEvents = [];

						$log.debug('error, GET ' + config.url, data, status, headers, config);
					});
			}
		}

		// normally we'd put some sort of check in place to wait for both values to change, but there's no need this time as the second
		// time fetchReport gets executed, it will be impossible for the $http request to have been fulfilled at that point.
		$scope.$watch('from', fetchReport);
		$scope.$watch('to', fetchReport);
	}
}());
