(function() {
	'use strict';

	angular.module('ipconfigure.orchid').controller('AdminSystemReportLiveCtrl', ['$scope', '$filter', '$http', '$interval', '$log', '$uibModal', 'BaseServiceUrl', 'StreamService', AdminSystemReportLiveCtrl]);

	function AdminSystemReportLiveCtrl($scope, $filter, $http, $interval, $log, $uibModal, BaseServiceUrl, StreamService) {
		var _statusInterval;

		$scope.activeStreams = StreamService.streams;
		$scope.activeStreamsMeta = {};

		function updateStreamStatus(streamId, activeStreamMeta) {
			if (activeStreamMeta.processingStatus !== true && activeStreamMeta.processStatusAfter < Date.now()) {
				activeStreamMeta.processingStatus = true;

				$http.get(BaseServiceUrl + '/service/streams/' + streamId + '/status')
					.success(function(status) {
						activeStreamMeta.status = status.streamState;
						activeStreamMeta.retryCount = typeof status.retryCount === 'number' ? status.retryCount : '?';

						if (status.lastArchiveWritten) {
							activeStreamMeta.lastArchiveWrittenIssue = status.lastArchiveWritten < Date.now() - 2 * 60 * 1000;
							activeStreamMeta.lastArchiveWrittenHumanized = $filter('timeDuration')(Date.now() - status.lastArchiveWritten, true, 1);
						}
						else {
							activeStreamMeta.lastArchiveWrittenIssue = true;
							activeStreamMeta.lastArchiveWrittenHumanized = 'unknown';
						}

						activeStreamMeta.processingStatus = false;
						activeStreamMeta.processStatusAfter = Date.now() + ((activeStreamMeta.status === 'running') ? 60000 : 2000);
					})
					.error(function() {
						activeStreamMeta.status = 'unable to retrieve';
						activeStreamMeta.lastArchiveWrittenHumanized = 'unknown';
						activeStreamMeta.lastArchiveWrittenIssue = false;
						activeStreamMeta.retryCount = 0;
						activeStreamMeta.processingStatus = false;
						activeStreamMeta.processStatusAfter = Date.now() + 2000;
					});
			}
		}

		$scope.previewStream = function(stream) {
			$uibModal.open({
				size: 'lg',
				templateUrl: '/views/stream-preview.html',
				controller: 'StreamPreviewCtrl',
				backdrop: true,
				resolve: {
					stream: function() {
						return stream;
					}
				}
			});
		};

		$scope.restartStream = function(stream) {
			var patchData = {
				primaryStream: {
					id: stream.id
				}
			};

			// say we're restarting and delay rechecking the status on this stream
			$scope.activeStreamsMeta[stream.id].status = 'restarting';
			$scope.activeStreamsMeta[stream.id].processStatusAfter = Date.now() + 1000 * 60;

			$http.patch(stream.cameraExpanded.instance.baseUrl + '/service/cameras/' + stream.camera.id, patchData)
				.success(function() {
					// for the stream to update during the next cycle
					$scope.activeStreamsMeta[stream.id].processStatusAfter = Date.now();
				})
				.error(function(data, status, headers, config) {
					$log.debug('error, PATCH /service/cameras/' + stream.camera.id, data, status, headers, config);
				});
		};

		$scope.verifyCamera = function(stream) {
			$scope.activeStreamsMeta[stream.id].processingVerify = true;

			$http.get(stream.cameraExpanded.instance.baseUrl + '/service/cameras/' + stream.camera.id + '/verify').success(function(response) {
				if (typeof response.success === 'string') {
					response.success = response.success === 'true';
				}

				$scope.activeStreamsMeta[stream.id].processingVerify = false;
				$scope.activeStreamsMeta[stream.id].verifyResult = response;

				if (response.success === false) {
					alert('Camera verification failed:\n\n' + response.message);
				}
			});
		};

		$scope.$watchCollection('activeStreams', function() {
			// build a list of activeStreams
			$scope.activeStreams.forEach(function(stream) {
				if ($scope.activeStreamsMeta.hasOwnProperty(stream.id) === false) {
					$scope.activeStreamsMeta[stream.id] = {
						processingVerify: false,
						verifyResult: false,
						status: '',
						processingStatus: false,
						processStatusAfter: Date.now()
					};
				}
			});
		});

		$scope.$on('$destroy', function() {
			$interval.cancel(_statusInterval);
		});

		// go into a loop refreshing stream status
		_statusInterval = $interval(function() {
			var streamId;

			for (streamId in $scope.activeStreamsMeta) {
				if ($scope.activeStreamsMeta.hasOwnProperty(streamId) === true) {
					updateStreamStatus(streamId, $scope.activeStreamsMeta[streamId]);
				}
			}
		}, 1000);

	}
}());
