(function() {
	'use strict';

	angular.module('ipconfigure.orchid').controller('AppCtrl', ['$scope', '$filter', '$http', '$uibModal', '$rootScope', '$timeout', 'AuthService', 'OrchidDiscoveryService', 'TimeDeltaService', AppCtrl]);

	function AppCtrl($scope, $filter, $http, $uibModal, $rootScope, $timeout, AuthService, OrchidDiscoveryService, TimeDeltaService) {
		var _fbgstCrashCheckTimer,
			_resizeTimer,
			_browser = ipConfigure.core.getBrowser(),
			_fbgst,
			licenseSessionStatusWatch;

		function _verifyBrowser() {
			// modern browser required
			switch (_browser.name) {
				case 'Chrome':
					return (_browser.version >= 34) ? '' : 'unsupported';
				case 'Firefox':
					return (_browser.version >= 33) ? '' : 'unsupported';
				case 'MSIE':
					return (_browser.version >= 11) ? '' : 'unsupported';
				case 'Opera':
					return (_browser.version >= 25) ? '' : 'unsupported';
				case 'Safari':
					return (_browser.version >= 8) ? '' : 'unsupported';
			}

			return 'unknown';
		}

		function _supportsFbgst(browser) {
			var supportedChromeVersion = (
				(browser.operatingSystem === 'Linux' && browser.version === 34) ||
				(browser.operatingSystem === 'Windows' && browser.version <= 44)
			);

			return browser.isMobile === false &&
				browser.isTablet === false &&
				browser.operatingSystem !== 'MacOS' &&
				(browser.name !== 'Chrome' || (browser.name === 'Chrome' && supportedChromeVersion === true));
		}

		function _verifyFbgst() {
			var param;

			if ($rootScope.compatibilityModeForced === true) {
				return true;
			}
			else {
				_fbgst = document.createElement('object');
				_fbgst.setAttribute('type', 'application/x-fbgst');
				_fbgst.setAttribute('id', 'fbgst');
				param = document.createElement('param');
				param.setAttribute('name', 'windowless');
				param.setAttribute('value', 'true');
				_fbgst.appendChild(param);
				document.body.appendChild(_fbgst);

				// FBGST minimum version check
				return !angular.isUndefined(_fbgst) && ipConfigure.core.compareVersions(_fbgst.version, window.requiredFbgstVersion);
			}
		}

		function _verifyPrerequisites(skip) {
			var browserResult = _verifyBrowser();

			if (skip !== true && browserResult !== '') {
				$scope.source = 'views/prerequisites/browser-' + browserResult + '.html';
			}
			else if (skip !== true &&
				_supportsFbgst(_browser) === true &&
				_verifyFbgst() === false) {
				$scope.source = 'views/prerequisites/fbgst.html';
			}
			else {
				AuthService.authenticateViaRememberCookie()
					.then(function() {
						$rootScope.$broadcast('logIn');
					})
					.catch(function(response) {
						$scope.source = 'views/login.html';

						if (typeof response !== 'object') {
							$scope.failure = 'An unexpected error occurred.';
						}
						else if (response && response.code !== 401) {
							$scope.failure = response.code + ': ' + response.reason;
						}
					});
			}
		}

		$rootScope.compatibilityMode = localStorage.compatibilityMode === 'true';
		$rootScope.compatibilityModeForced = false;

		$scope.currentInstance = OrchidDiscoveryService.currentInstance;
		$scope.adminMode = false;
		$scope.loggedIn = false;
		$scope.loggingOut = false;
		$scope.loggedOutMessage = '';
		$scope.role = '';
		$scope.source = 'views/loading.html';
		$scope.isAdmin = false;
		$scope.clientServerTimeDifference = 0;

		$scope.configure = function() {
			$scope.adminMode = !$scope.adminMode;

			if ($scope.adminMode === false) {
				// this doesn't belong here, but unfortunately once FBGST gets hidden and shown again (style.display), it
				// seems to stop rendering.  to get around that, when configuration opens, i'm hiding #main by setting its
				// height to 0.  this has the adverse effect of if the browser is resized while it admin mode, shit goes awry.
				$timeout(function() {
					var ev = document.createEvent('Event');
					ev.initEvent('resize', true, true);
					window.dispatchEvent(ev);
				});
			}
		};

		$scope.logOut = function(reason) {
			if ($scope.loggingOut === false) {
				$scope.adminMode = false;
				$scope.loggingOut = true;

				AuthService.logOut()
						.then(function() {
							$scope.loggedIn = false;
							$scope.source = 'views/login.html';

							$scope.loggedOutMessage = reason;
						})
						.catch(function() {
							// completely unexpected -- all bets are off, just bail
							window.alert('An unexpected error occurred.');

							location.reload();
						});
			}
		};

		$scope.forceCompatibilityMode = function() {
			$rootScope.compatibilityMode = true;
			$rootScope.compatibilityModeForced = true;

			_verifyPrerequisites(false);
		};

		$scope.licenseWarning = function() {
			$uibModal.open({
				size: 'lg',
				templateUrl: '/views/degraded-warning.html',
				controller: 'DegradedWarningCtrl',
				backdrop: true,
				resolve: {
					licenseSession: function() {
						return $scope.currentInstance.licenseSession;
					}
				}
			});
		};

		$scope.timeWarning = function() {
			TimeDeltaService.fetchDifferenceInfo().then(function(differenceInfo) {
				var humanizedDifference;

				if (differenceInfo.initialized === false) {
					// this is not likely to ever occur
					alert('Time check uninitialized.  If the client/server time are mismatched, you may experience playback issues.');
				}
				else if (differenceInfo.failed === true) {
					alert('Unable to verify server time.  If the client/server time are mismatched, you may experience playback issues.');
				}
				else {
					humanizedDifference = $filter('timeDuration')(Math.abs(differenceInfo.diff), true, 1);

					alert('The client time and server time are off by ' + humanizedDifference + '.\n\n' +
					'You should seek to correct this using an NTP server, as it may cause playback issues.\n\n' +
					'Client: ' + $filter('date')(differenceInfo.client, 'MMMM d, yyyy h:mm:ss a') + '\n' +
					'Server: ' + $filter('date')(differenceInfo.server, 'MMMM d, yyyy h:mm:ss a'));
				}
			});
		};

		$rootScope.$on('logIn', function() {
			var user = AuthService.getAuthenticatedUser();

			if (location.href.indexOf('configuration.html') > -1) {
				// if they're on the configuration page (because ipad, see fixAwfulAppleProducts::config-link), then auto-set adminMode to true
				$scope.adminMode = true;
			}

			$scope.source = 'views/soul.html';

			$scope.loggedIn = true;
			$scope.loggingOut = false;

			$scope.role = user.role;
			$scope.username = user.username;

			$scope.isAdmin = $scope.role === 'Administrator';
		});

		$rootScope.$on('logOut', function(e, reason) {
			$scope.logOut(reason);
		});

		$rootScope.$on('prerequisiteSatisfied', function(event, skip) {
			_verifyPrerequisites(skip);
		});

		$scope.$on('$destroy', function() {
			$timeout.cancel(_fbgstCrashCheckTimer);
		});

		// prerequisite checks
		_verifyPrerequisites(false);

		// force compatibility mode for when FBGST is not supported
		if (_supportsFbgst(_browser) === false) {
			$rootScope.compatibilityMode = true;
			$rootScope.compatibilityModeForced = true;
		}

		// kick off the ping timer so we can detect if/when the orchid server shuts down
		// there's no real need to ever destroy the ping timer
		(function _ping() {
			$http.get('/ping')
				.success(function() {
					$timeout(_ping, 5000);
				})
				.error(function() {
					$timeout(_ping, 5000);
				});
		}());

		// avoid duplication of this logic everywhere by $broadcasting a resize event for the entire application
		window.addEventListener('resize', function() {
			if (_resizeTimer) {
				clearTimeout(_resizeTimer);
			}
			_resizeTimer = setTimeout(function() {
				$rootScope.$broadcast('resize');
			}, 150);
		}, true);

		// if fbgst is initially available, kick off a timer that will refresh the page when gst
		// crashes, otherwise the pre-req check in AppCtrl should catch it
		if (_fbgst && typeof _fbgst.version !== 'undefined') {
			(function _fbgstCrashCheck() {
				if (typeof _fbgst.version === 'undefined') {
					window.location.reload();
				}

				_fbgstCrashCheckTimer = $timeout(_fbgstCrashCheck, 5000);
			}());
		}

		// make the degraded state more dire
		licenseSessionStatusWatch = $scope.$watch('currentInstance.licenseSession.state.status', function(status) {
			if (angular.isUndefined(status) === false) {
				if (status === 'degraded') {
					$scope.licenseWarning();
				}

				// no need to listen anymore
				licenseSessionStatusWatch();
			}
		});
	}
}());
