(function() {
	'use strict';

	angular.module('ipconfigure.orchid.directive').directive('bag', ['$log', '$rootScope', '$timeout', 'StreamService', 'StageService', bag]);

	function bag($log, $rootScope, $timeout, StreamService, StageService) {
		return {
			restrict: 'E',
			replace: true,
			scope: {
				contextClick: '&',
				addPlayer: '&'
			},
			template: ' ' +
			'<section>' +
			'  <div class="list-container">' +
			'    <div class="bag-item" ng-repeat="stream in streams | orderBy:\'cameraExpanded.name\'" ng-dblclick="handleDblClick(stream, $event)" title="c={{stream.camera.id}} s={{stream.id}}">' +
			'      <video-preview stream="stream"></video-preview>' +
			'      <div class="video-preview-label">' +
			'           <div title="{{stream.cameraExpanded.name}}">{{stream.cameraExpanded.name}}</div>' +
			'           <div title="{{stream.name}}">{{stream.name}}</div>' +
			'       </div>' +
			'    </div>' +
			'    <div ng-show="processing === true" style="text-align: center; cursor: default;">' +
			'      <i class="fa fa-spinner fa-spin fa-3x" style="color: #8f8f8f;"></i>' +
			'      <div style="color: #afafaf;">Loading streams...</div>' +
			'    </div>' +
			'    <div ng-show="processing === false && streams.length === 0" style="text-align: center; cursor: default;">' +
			'      <i class="fa fa-frown-o fa-4x" style="color: #8f8f8f;"></i>' +
			'      <div style="color: #afafaf;">There are no configured cameras at this time</div>' +
			'    </div>' +
			'  </div>' +
			'  <img class="toggle" ng-click="toggleContext()" src="/img/toggle-on.svg" />' +
			'</section>',
			link: function(scope, element) {
				var _d3Element = d3.select(element[0]),
					_toggleElement = _d3Element.select('.toggle'),
					_containerElement = _d3Element.select('.list-container'),
					_dragFloater,
					_refreshTimer;

				function _createDragFloater() {
					if (!_dragFloater) {
						_dragFloater = document.createElement('canvas');
						_dragFloater.style.position = 'fixed';
						_dragFloater.style.zIndex = 999;
						_dragFloater.style.top = 0;
						_dragFloater.style.width = 0;
						_dragFloater.style.height = 0;
						_dragFloater.style.display = 'none';
						document.body.appendChild(_dragFloater);
					}
				}

				function _findRootElement(containerElement, descendantElement) {
					if (descendantElement === containerElement || descendantElement.parentNode === containerElement) {
						return descendantElement;
					}
					else {
						return _findRootElement(containerElement, descendantElement.parentNode);
					}
				}

				function _findStreamBySource(source) {
					var i,
						len;

					for (i = 0, len = scope.streams.length; i < len; i += 1) {
						if (scope.streams[i].liveStream === source) {
							return scope.streams[i];
						}
					}

					return false;
				}

				function _restoreStreamsToStage() {
					var streamLookup = {};

					// done loading streams, let's import them
					scope.streams.forEach(function(stream) {
						streamLookup[stream.id] = stream;
					});

					StageService.reloadStageConfiguration(function(streamId, cell) {
						if (streamLookup[streamId]) {
							// further attempts to avoid width/height: 0 on frame puller
							$timeout(function() {
								_addPlayer(streamLookup[streamId], cell, true);
							});
						}
						else {
							// handle an old stream that no longer exists
							StageService.releaseCell(cell, true);
						}
					});
				}

				function _onDragStart(d) {
					var rootElement = _findRootElement(d.container, d3.event.sourceEvent.target),
						thumb = rootElement.firstElementChild.firstElementChild;

					ipConfigure.core.some(rootElement.querySelectorAll('img'), function(img) {
						if (img.style.display !== 'none') {
							thumb = img;
							return false;
						}
					});

					// only let them drag right if they've dragged the preview image
					if (d3.event.sourceEvent.target.tagName === 'IMG') {
						d.stream = angular.element(d3.event.sourceEvent.target).scope().stream;
						d.target = rootElement;

						if (window.TouchEvent && d3.event.sourceEvent instanceof window.TouchEvent) {
							// if the user is using touch,
							d.offsetX = d3.event.sourceEvent.touches[0].pageX + 82;
							d.offsetY = d3.event.sourceEvent.touches[0].pageY - d3.event.sourceEvent.target.offsetTop - 36 - element.prop('offsetTop');
						}
						else {
							// normally we'd rely on d3.event.sourceEvent.offsetX, but it can't be trusted in FireFox
							d.offsetX = d3.event.sourceEvent.clientX - d3.event.sourceEvent.target.offsetLeft + 200;
							d.offsetY = d3.event.sourceEvent.clientY - d3.event.sourceEvent.target.offsetTop - 36 - element.prop('offsetTop');
						}

						_dragFloater.style.opacity = 0.75;
						_dragFloater.style.width = '';
						_dragFloater.style.height = '';
						_dragFloater.width = thumb.clientWidth;
						_dragFloater.height = thumb.clientHeight;
						_dragFloater.getContext('2d').drawImage(thumb, 0, 0, thumb.clientWidth, thumb.clientHeight);
					}
					else {
						d.scroll = true;
					}
				}

				function _onDrag(d) {
					if (!d.scroll && !d.drag) {
						// we've not yet decided what to do, let's start by saving the coordinates... we might want to do this in dragstart, but
						// it may prove challenging to get the same coordinates between MouseEvent and TouchEvent
						if (!d.originalCoordinates) {
							d.originalCoordinates = {
								x: d3.event.x,
								y: d3.event.y
							};
						}
						else {
							if (d3.event.x >= (d.originalCoordinates.x + 60)) {
								d.drag = true;
							}
							else if (d3.event.y >= d.originalCoordinates.y + 60 || d3.event.y <= d.originalCoordinates.y - 60) {
								d.scroll = true;
							}
						}
					}

					if (d.scroll) {
						d.container.scrollTop -= d3.event.dy;
					}
					else if (d.drag) {
						d.dragged = true;

						_dragFloater.style.display = '';
						_dragFloater.style.left = (d3.event.x - d.offsetX) + 'px';
						_dragFloater.style.top = (d3.event.y - d.offsetY - d.container.scrollTop) + 'px';

						if (window.TouchEvent && d3.event.sourceEvent instanceof window.TouchEvent) {
							d.pageX = d3.event.sourceEvent.touches[0].pageX;
							d.pageY = d3.event.sourceEvent.touches[0].pageY;
						}
						else {
							d.pageX = d3.event.sourceEvent.pageX;
							d.pageY = d3.event.sourceEvent.pageY;
						}

						// only highlight the cell if they're not over the bag
						if (d.pageX > 190) {
							StageService.dragOver(d.pageX, d.pageY);
						}
						else {
							StageService.dragClear();
						}
					}
				}

				function _onDragEnd(d) {
					if (d.target && d.dragged) {
						if (d.pageX < 190) {
							// animate the drag floater back to its home
							$log.debug('dragend', d.target.offsetTop, d.target.offsetLeft);

							_sendDragFloater(d.target.firstElementChild.firstElementChild.offsetLeft - 100, d.target.offsetTop + 36 - d.container.scrollTop);

							StageService.drop(true);
						}
						else {
							StageService.claimCellByCoordinates(d.pageX, d.pageY, function(cell) {
								if (cell !== false) {
									// if the cell the user dropped on has been modified (assumed to be by us), then just put it in the next available slot
									if (cell.element.node().firstElementChild) {
										// i don't really like that I'm referencing claimed here, or how I'm going about getting the playerId
										$rootScope.$broadcast('removePlayer', angular.element(cell.element.node().firstElementChild).scope().playerId, true);
									}

									_addPlayer(d.stream.liveStream, cell);

									StageService.drop(false);
								}
								else {
									// animate back home
									_sendDragFloater(d.target.firstElementChild.firstElementChild.offsetLeft - 100, d.target.offsetTop + 36 - d.container.scrollTop);

									StageService.drop(true);
								}
							});
						}
					}

					delete d.dragged;
					delete d.drag;
					delete d.originalCoordinates;
					delete d.scroll;
					delete d.stream;
					delete d.target;
					d3.event.sourceEvent.stopPropagation();
				}

				function _addPlayer(sourceOrStream, cell, dontAnimate) {
					var stream = sourceOrStream,
						constrainedDimensions,
						constrainedWidth,
						constrainedHeight,
						constrainedTop,
						constrainedLeft;

					if (typeof sourceOrStream === 'string') {
						stream = _findStreamBySource(sourceOrStream);
					}

					if (dontAnimate === true) {
						scope.addPlayer({
							stream: stream,
							container: cell
						});
					}
					else {
						// fill cell with centered drag floater
						if (angular.isUndefined(stream.aspectRatio) === false) {
							constrainedDimensions = ipConfigure.core.fillRectangle(cell.left, cell.top, cell.width, cell.height, stream.aspectRatio[0], stream.aspectRatio[1]);
							constrainedWidth = constrainedDimensions.width;
							constrainedHeight = constrainedDimensions.height;
							constrainedTop = constrainedDimensions.top + 36;
							constrainedLeft = constrainedDimensions.left;
						}
						else {
							constrainedWidth = cell.width;
							constrainedHeight = cell.height;
							constrainedTop = (cell.top + 36);
							constrainedLeft = cell.left + ((cell.width - constrainedWidth) / 2);
						}

						d3.select(_dragFloater)
							.style('opacity', 0.75)
							.transition()
							.duration(333)
							.style('opacity', 0.25)
							.style('top', constrainedTop + 'px')
							.style('left', constrainedLeft + 'px')
							.style('width', constrainedWidth + 'px')
							.style('height', constrainedHeight + 'px')
							.each('end', function() {
								_dragFloater.style.display = 'none';
							});

						// and fire our addPlayer event after the animation is expected to complete
						$timeout(function() {
							scope.addPlayer({
								stream: stream,
								container: cell
							});
						}, 333);
					}
				}

				function _sendDragFloater(x, y) {
					d3.select(_dragFloater)
						.style('opacity', 0.75)
						.transition()
						.duration(333)
						.style('opacity', 0)
						.style('left', x + 'px')
						.style('top', y + 'px')
						.each('end', function() {
							_dragFloater.style.display = 'none';
						});
				}

				function _init() {
					var dragBehavior;

					// configure the drag/drop behavior
					dragBehavior = d3.behavior.drag()
						.on('dragstart', _onDragStart)
						.on('drag', _onDrag)
						.on('dragend', _onDragEnd);

					// wire up the drag behavior to the bag
					_containerElement
						.call(dragBehavior)
						.data([
							{ container: _containerElement.node(), x: 0, y: 0 }
						]);

					_createDragFloater();
				}

				scope.state = 'closed';
				scope.processing = true;
				scope.streams = StreamService.streams;
				scope.streamServiceState = StreamService.state;

				scope.$watch('streamServiceState.currentState', function(state) {
					// TODO this approach will not work for restoring streams on mesh, because the state will become 'loaded' when an instance first provides a list of cameras
					if (state === 'loaded') {
						scope.processing = false;

						// using $timeout to allow stage to render and sizes to be set
						$timeout(_restoreStreamsToStage);
					}
				});

				scope.$on('$destroy', function() {
					// clean up your stuff
					if (_refreshTimer) {
						$timeout.cancel(_refreshTimer);
					}
				});

				scope.$on('bag-force-close', function() {
					scope.toggleContext(true);
				});

				scope.toggleContext = function(forceClose) {
					if (scope.state === 'closed' && forceClose !== true) {
						_d3Element.style('left', '-289px').transition().ease('elastic').duration(600).style('left', '-100px');
						//							_toggleElement.style('background-position', '0 -33px');
						_toggleElement.attr('src', '/img/toggle-off.svg');
						scope.state = 'open';
						scope.$broadcast('video-preview-play');
					}
					else {
						_d3Element.transition().ease('elastic').duration(600).style('left', '-289px');
						//							_toggleElement.style('background-position', '0 0');
						_toggleElement.attr('src', '/img/toggle-on.svg');
						$timeout(function() {
							scope.state = 'closed';
							scope.$broadcast('video-preview-stop');
						}, 601);
					}

					scope.contextClick();
				};

				scope.handleDblClick = function(stream, $event) {
					StageService.obtainContainer(function(cell) {
						var sourceElement = $event.target,
							thumbElement,
							floater = d3.select(_dragFloater);

						if (sourceElement.className.indexOf('video-preview-label') > -1) {
							sourceElement = sourceElement.parentNode.firstElementChild;
						}
						else {
							// recursively find parent with 'bag-item' class
							while (sourceElement.className.indexOf('bag-item') === -1) {
								sourceElement = sourceElement.parentNode;
							}
						}

						thumbElement = sourceElement.firstElementChild.firstElementChild;

						// position the drag floater over top of the cell the user double clicked
						floater
							.attr('src', stream.thumb)
							.style('opacity', 0.75)
							.style('left', (thumbElement.offsetLeft - 100) + 'px')
							.style('top', (thumbElement.offsetTop + 36 - _containerElement.node().scrollTop) + 'px')
							.style('width', thumbElement.clientWidth + 'px')
							.style('height', thumbElement.clientHeight + 'px')
							.style('display', '');

						_addPlayer(stream, cell);
					});
				};

				_init();
			}
		};
	}
}());
