(function() {
	'use strict';

	angular.module('ipconfigure.orchid.directive').directive('digitalPtz', ['$log', '$rootScope', '$timeout', digitalPtz]);

	function digitalPtz($log, $rootScope, $timeout) {
		return {
			restrict: 'A',
			scope: {
				digitalPtz: '=',
				digitalPtzSettings: '='
			},
			link: function(scope, wrappedElement) {
				var zoomContainer = wrappedElement[0],
					zoomTarget,
					zoomAdjustment = 0.10,
					width,
					height,
					bgWidth,
					bgHeight,
					bgPosX = 0,
					bgPosY = 0,
					lastMouseEvent;

				// give time for everything to load (mainly the video component)
				$timeout(init, 100);

				function init() {
					zoomTarget = zoomContainer.children[0];

					baselinePositionData();
					reset();

					// handle resize
					scope.$on('resize.stage', function() {
						baselinePositionData();
					});

					// handle mousewheel
					wrappedElement.bind('mousewheel DOMMouseScroll', onWheel);

					// make the background draggable
					wrappedElement.bind('mousedown', onMouseDown);

					scope.$on('$destroy', function() {
						wrappedElement.unbind('mousewheel DOMMouseScroll', onWheel);
						wrappedElement.unbind('mousedown', onMouseDown);
					});
				}

				function reset() {
					bgWidth = width;
					bgHeight = height;
					bgPosX = bgPosY = 0;
					updateBgStyle();
				}

				function updateBgStyle() {
					var zoomValue;

					if (bgPosX > 0) {
						bgPosX = 0;
					}
					else if (bgPosX < width - bgWidth) {
						bgPosX = width - bgWidth;
					}

					if (bgPosY > 0) {
						bgPosY = 0;
					}
					else if (bgPosY < height - bgHeight) {
						bgPosY = height - bgHeight;
					}

					zoomValue = Math.round(bgWidth / width * 100);

					zoomTarget.style.width = zoomValue + '%';
					zoomTarget.style.height = zoomValue + '%';
					zoomTarget.style.left = -(Math.abs(bgPosX) / width * 100) + '%';
					zoomTarget.style.top = -(Math.abs(bgPosY) / height * 100) + '%';

					scope.digitalPtz = zoomValue;
					scope.digitalPtzSettings = { zoom: zoomValue, left: -(Math.abs(bgPosX) / width * 100), top: -(Math.abs(bgPosY) / height * 100) };

					scope.$parent.$digest();

					$log.debug('wxh', bgWidth, bgHeight);
				}

				function onWheel(e) {
					var deltaY = 0,
						newBg = {},
						rect,
						offsetX,
						offsetY,
						bgCursorX,
						bgCursorY,
						bgRatioX,
						bgRatioY;

					e.preventDefault();

					if (e.deltaY) {
						deltaY = e.deltaY;
					}
					else if (e.wheelDelta) {
						deltaY = -e.wheelDelta;
					}
					else if (e.detail) {
						deltaY = e.detail;
					}

					// calculate the target element's position relative to the document, and subtract that from the cursor's position relative to the document
					rect = zoomContainer.parentNode.getBoundingClientRect();
					offsetX = e.pageX - rect.left + document.body.scrollLeft;
					offsetY = e.pageY - rect.top + document.body.scrollTop;

					// record the offset between the bg edge and cursor:
					bgCursorX = offsetX - bgPosX;
					bgCursorY = offsetY - bgPosY;

					// use the previous offset to get the percent offset between the bg edge and cursor
					bgRatioX = bgCursorX / bgWidth;
					bgRatioY = bgCursorY / bgHeight;

					if (deltaY < 0) {
						newBg.width = bgWidth + bgWidth * zoomAdjustment;
						newBg.height = bgHeight + bgHeight * zoomAdjustment;
					}
					else {
						newBg.width = bgWidth - bgWidth * zoomAdjustment;
						newBg.height = bgHeight - bgHeight * zoomAdjustment;
					}

					// adjust width and height if it has exceeds our ceiling
					if ($rootScope.compatibilityMode !== true && newBg.width > 2700) {
						newBg.height -= (newBg.width - 2700) / newBg.width * newBg.height;
						newBg.width = 2700;
					}
					else if ($rootScope.compatibilityMode !== true && newBg.height > 2500) {
						// adjust width and height
						newBg.width -= (newBg.height - 2500) / newBg.height * newBg.width;
						newBg.height = 2500;
					}

					// limit the size of the zoom if fbvlc is being used
					if ($rootScope.compatibilityMode === true || (newBg.width <= 2700 && newBg.height <= 2500)) {
						// update the bg size
						bgWidth = newBg.width;
						bgHeight = newBg.height;

						// take the percent offset and apply it to the new size
						bgPosX = offsetX - (bgWidth * bgRatioX);
						bgPosY = offsetY - (bgHeight * bgRatioY);

						// prevent zooming out beyond the starting size
						if (bgWidth <= width || bgHeight <= height) {
							reset();
						}
						else {
							updateBgStyle();
						}
					}
				}

				function onMouseDown(e) {
					var zoomValue = Math.round(bgWidth / width * 100);

					if (Math.round(zoomValue) !== 100) {
						e.preventDefault();
						e.stopPropagation();
						lastMouseEvent = e;

						document.addEventListener('mousemove', onDrag);
						document.addEventListener('mouseup', onDrop);
					}
				}

				function onDrag(e) {
					e.preventDefault();
					bgPosX += (e.pageX - lastMouseEvent.pageX);
					bgPosY += (e.pageY - lastMouseEvent.pageY);
					lastMouseEvent = e;
					updateBgStyle();
				}

				function onDrop() {
					document.removeEventListener('mousemove', onDrag);
					document.removeEventListener('mouseup', onDrop);
				}

				function baselinePositionData() {
					var zoomValue = parseInt(zoomTarget.style.width, 10) / 100;

					width = zoomContainer.parentNode.clientWidth;
					height = zoomContainer.parentNode.clientHeight;

					// if the styles have been set, adjust our internal variables accordingly
					if (zoomTarget.style.width !== '') {
						bgWidth = width * zoomValue;
						bgHeight = height * zoomValue;
						bgPosX = width * (parseInt(zoomTarget.style.left, 10) / 100);
						bgPosY = height * (parseInt(zoomTarget.style.top, 10) / 100);

						$log.debug('bg', bgWidth, bgHeight, bgPosX, bgPosY);
					}
					else {
						// no zoom yet, normalize the values
						bgWidth = width;
						bgHeight = height;
						bgPosX = 0;
						bgPosY = 0;
					}
				}
			}
		};
	}
}());
