(function() {
	'use strict';

	angular.module('ipconfigure.orchid.directive').directive('diskUtesBar', ['$http', '$timeout', 'BaseServiceUrl', diskUtesBar]);

	function diskUtesBar($http, $timeout, BaseServiceUrl) {
		return {
			restrict: 'E',
			scope: {
				forceBreak: '='
			},
			template: ' ' +
				'<div class="disk-utes-bar">' +
				'   <uib-progress animate="true" progressbar-threshold="diskUtilization.thresholdPercentage" class="progress-striped" ng-class="{ active: diskUtilization.active, \'progress-paused\': !diskUtilization.animate }">' +
				'       <uib-bar value="diskUtilization.percentageUsedByOthers" type="dull"></uib-bar>' +
				'       <uib-bar value="diskUtilization.percentageUsedByOrchid" type="{{diskUtilization.type}}"></uib-bar>' +
				'   </uib-progress>' +
				'   <div class="sub-description" ng-visible="!diskUtilization.active">' +
				'       <span class="legend-box progress-bar-dull"></span>' +
				'       {{diskUtilization.usedByOthers | bytes}} of unusable space' +
				'       <br ng-class="{ \'visible-xs-block\': !forceBreak }">' +
				'       <span class="legend-box progress-bar-success"></span>' +
				'       {{diskUtilization.usedByOrchid | bytes}} of video data' +
				'       <br ng-class="{ \'visible-xs-block\': !forceBreak }">' +
				'       <span class="legend-bar"></span>' +
				'       cleaner threshold at {{diskUtilization.threshold | bytes}}' +
				'       <br ng-class="{ \'visible-xs-block\': !forceBreak }">' +
				'       <span class="legend-box legend-container"></span>' +
				'       {{diskUtilization.free | bytes}} free out of {{diskUtilization.capacity | bytes}}' +
				'   </div>' +
				'</div>',
			link: function(scope) {
				scope.diskUtilization = {
					animate: false,
					active: true,
					type: 'info',
					usedByOrchid: 0,
					usedByOthers: 0,
					percentageUsedByOrchid: 100,
					percentageUsedOthers: 0,
					thresholdPercentage: 100
				};

				$http.get(BaseServiceUrl + '/service/server/utilization/disk').success(function(response) {
					scope.diskUtilization.capacity = response.storage.capacity;
					scope.diskUtilization.free = response.storage.free;
					scope.diskUtilization.threshold = response.cleanerThreshold + response.storage.unavailable;

					scope.diskUtilization.usedByOrchid = response.archivesSize;
					scope.diskUtilization.usedByOthers = response.storage.unavailable;

					scope.diskUtilization.animate = true;

					// allow for the progress bar to animate nicely
					$timeout(function() {
						scope.diskUtilization.active = false;
						scope.diskUtilization.type = 'success';

						scope.diskUtilization.percentageUsedByOrchid = response.archivesSize / response.storage.capacity * 100;
						scope.diskUtilization.percentageUsedByOthers = response.storage.unavailable / response.storage.capacity * 100;

						scope.diskUtilization.thresholdPercentage = (response.cleanerThreshold + response.storage.unavailable) / response.storage.capacity * 100;
					}, 50);
				});
			}
		};
	}
}());
