(function() {
	'use strict';

	angular.module('ipconfigure.orchid.directive').directive('motionDetectionContainer', ['$timeout', motionDetectionContainer]);

	function motionDetectionContainer($timeout) {
		return {
			restrict: 'E',
			scope: {
				background: '@',
				configuration: '=',
				resolution: '='
			},
			replace: true,
			template: ' ' +
				'<div style="position: relative">' +
				'   <img class="image" ng-src="{{background}}" style="width: 100%" />' +
				'   <div class="area" ng-style="{ width: areaDimension + \'px\', height: areaDimension + \'px\' }"></div>' +
				'   <div class="zone" ng-repeat="zone in detectionZonesCopy" client-id="{{zone.clientId}}" ng-style="{ top: zone.uiY + \'px\',  left: zone.uiX + \'px\',  width: zone.uiW + \'px\', height: zone.uiH + \'px\' }"></div>' +
				'</div>',
			link: function(scope, element) {
				var _d3Element = d3.select(element[0]),
					_detectionAreaScale = d3.scale.linear().domain([0, 100]),
					_widthScale = d3.scale.linear(),
					_heightScale = d3.scale.linear(),
					_zIndex = 500,
					_resizePromise,
					clientIdSeed = 1;

				angular.element(element[0].firstElementChild).bind('load', function() {
					_updateScales();
					_updateUiZonePositions();
					_styleMinimumDetectionArea();
				});

				scope.$on('resize', function() {
					if (Array.isArray(scope.detectionZonesCopy) === true) {
						$timeout.cancel(_resizePromise);

						_resizePromise = $timeout(function() {
							// TODO dragend should handle this
//								scope.detectionZonesCopy.forEach(function (zone) {
//									zone.x = _widthScale.invert(zone.uiX);
//									zone.y = _heightScale.invert(zone.uiY);
//									zone.w = _widthScale.invert(zone.uiW);
//									zone.h = _heightScale.invert(zone.uiH);
//								});

							_updateScales();
							_updateUiZonePositions();
						}, 100);
					}
				});

				_d3Element.on('dblclick', function() {
					var clientIdAttr = d3.event.target.attributes['client-id'];

					if (clientIdAttr) {
						scope.detectionZonesCopy.some(function(zone, i) {
							if (zone.clientId === parseInt(clientIdAttr.value, 10)) {
								scope.detectionZonesCopy.splice(i, 1);
								return true;
							}
						});

						scope.$apply();
					}
				});

				_d3Element.data([
					{}
				]);

				_d3Element.call(d3.behavior.drag()
					.on('dragstart', _onDragStart)
					.on('drag', _onDrag)
					.on('dragend', _onDragEnd));

				scope.$watch('configuration.detectionZones', function(newValue) {
					if (typeof newValue !== 'undefined') {
						scope.detectionZonesCopy = angular.copy(scope.configuration.detectionZones);

						scope.detectionZonesCopy.forEach(function(zone) {
							zone.clientId = clientIdSeed++;

							_setUiZonePositions(zone);
						});
					}
					else {
						scope.detectionZonesCopy = [];
					}
				});

				scope.$watch('configuration.minimumDetectionArea', _styleMinimumDetectionArea);

				function _updateScales() {
					_widthScale.domain([0, parseInt(scope.resolution.split('x')[0], 10)]);
					_heightScale.domain([0, parseInt(scope.resolution.split('x')[1], 10)]);
					_widthScale.range([0, element[0].firstElementChild.clientWidth]);
					_heightScale.range([0, element[0].firstElementChild.clientHeight]);
				}

				function _setUiZonePositions(zone) {
					zone.uiX = _widthScale(zone.x);
					zone.uiY = _heightScale(zone.y);
					zone.uiW = _widthScale(zone.w);
					zone.uiH = _heightScale(zone.h);
				}

				function _updateUiZonePositions() {
					scope.detectionZonesCopy.forEach(function(zone) {
						_setUiZonePositions(zone);
					});
				}

				function _styleMinimumDetectionArea() {
					if (typeof scope.configuration !== 'undefined') {
						_detectionAreaScale.range([0, Math.min(element[0].clientWidth, element[0].clientHeight)]);
						scope.areaDimension = _detectionAreaScale(scope.configuration.minimumDetectionArea);
					}
				}

				function _onDragStart(d) {
					var clientId;

					//console.log('dragstart');

					d3.event.sourceEvent.preventDefault();
					d3.event.sourceEvent.stopPropagation();

					d.target = d3.select(d3.event.sourceEvent.target);
					d.action = d.target.attr('class');

					if (d.action.indexOf('zone') > -1) {
						d.action = 'zone';
					}
					else if (d.action.indexOf('area') > -1) {
						d.action = 'area';
					}
					else if (d.action.indexOf('image') > -1) {
						d.action = 'image';
					}

					switch (d.action) {
						case 'area':
						case 'image':
							// TODO if you want this to work for touch, you need to do some calculations.. TouchEvent provides
							// TODO clientX, clientY, pageX, and pageY which correspond to where the user clicked on the screen.
							// TODO it's annoying because it's based on viewport width and height, as opposed to docment width and height,
							// TODO so you have to figure out how far down the page the admin container is scrolled, which doesn't
							// TODO make this directive very reusable.
//								console.log('x y (1)', d3.event.sourceEvent.offsetX, d3.event.sourceEvent.offsetY);
//								if (d3.event.sourceEvent.changedTouches) {
//									console.log('x y (2)', d3.event.sourceEvent.changedTouches[0].clientX, d3.event.sourceEvent.changedTouches[0].clientY);
//									console.log(d3.event.sourceEvent.changedTouches[0]);
//								}
//								console.log(element[0]);
//								console.log('x y (3)', element[0].offsetLeft, element[0].offsetTop);
//								console.log('x y (4)', element[0].parentNode.offsetLeft, element[0].parentNode.offsetTop);

							d.coordinates = {
								x1: d3.event.sourceEvent.offsetX + 15,
								y1: d3.event.sourceEvent.offsetY
							};
							break;
						case 'zone':
							clientId = parseInt(d.target.attr('client-id'), 10);

							// set d.zone look up proper values based on clientId
							scope.detectionZonesCopy.some(function(zone) {
								if (zone.clientId === clientId) {
									d.zone = zone;
									return true;
								}
							});

							d.target.style('z-index', ++_zIndex);
							break;
					}
				}

				function _onDrag(d) {
					//console.log('drag');

					d3.event.sourceEvent.preventDefault();
					d3.event.sourceEvent.stopPropagation();

					//console.log('drag action', d.action);

					d.dragged = true;

					switch (d.action) {
						case 'area':
						case 'image':
							if (d.created !== true) {
								d.zone = {
									uiX: d.coordinates.x1 - 15,
									uiY: d.coordinates.y1,
									uiW: d3.event.dx,
									uiH: d3.event.dy,
									clientId: clientIdSeed++
								};
								scope.detectionZonesCopy.push(d.zone);
								d.created = true;
							}
							else {
								d.coordinates.x2 = Math.max(d3.event.x, 15);
								d.coordinates.y2 = Math.max(d3.event.y, 0);

								d.zone.uiX = ((d.coordinates.x1 < d.coordinates.x2) ? d.coordinates.x1 - 15 : d.coordinates.x2 - 15);
								d.zone.uiY = ((d.coordinates.y1 < d.coordinates.y2) ? d.coordinates.y1 : d.coordinates.y2);
								d.zone.uiW = ((d.coordinates.x1 > d.coordinates.x2) ? d.coordinates.x1 - d.coordinates.x2 : d.coordinates.x2 - d.coordinates.x1);
								d.zone.uiH = ((d.coordinates.y1 > d.coordinates.y2) ? d.coordinates.y1 - d.coordinates.y2 : d.coordinates.y2 - d.coordinates.y1);

								d.zone.uiW = Math.min(d.zone.uiW, element[0].firstElementChild.clientWidth - d.zone.uiX);
								d.zone.uiH = Math.min(d.zone.uiH, element[0].firstElementChild.clientHeight - d.zone.uiY);
							}

							break;
						case 'zone':
							d.zone.uiX += d3.event.dx;
							d.zone.uiY += d3.event.dy;

							d.zone.uiX = Math.min(Math.max(0, d.zone.uiX), Math.floor(_widthScale.range()[1]) - d.zone.uiW);
							d.zone.uiY = Math.min(Math.max(0, d.zone.uiY), Math.floor(_heightScale.range()[1]) - d.zone.uiH);

							break;
					}

					scope.$apply();
				}

				function _onDragEnd(d) {
					if (d.dragged === true) {
						//console.log('dragend');

						d3.event.sourceEvent.preventDefault();
						d3.event.sourceEvent.stopPropagation();

						switch (d.action) {
							case 'area':
							case 'image':
								if (d.zone.uiW < 20 || d.zone.uiH < 20) {
									scope.detectionZonesCopy.some(function(zone, i) {
										if (d.zone.clientId === zone.clientId) {
											scope.detectionZonesCopy.splice(i, 1);
											return true;
										}
									});
								}
								else {
									_updateScales();

									d.zone.x = _widthScale.invert(d.zone.uiX);
									d.zone.y = _heightScale.invert(d.zone.uiY);
									d.zone.w = _widthScale.invert(d.zone.uiW);
									d.zone.h = _heightScale.invert(d.zone.uiH);

									// TODO if a new rect was drawn, back fill it into scope.configuration.detectionZones
								}
								break;
							case 'zone':
								// TODO update scope.configuration.detectionZones

								break;
						}

						delete d.created;
						delete d.dragged;
						delete d.x;
						delete d.y;
						delete d.w;
						delete d.h;
						delete d.zone;
					}

					delete d.action;
					delete d.coordinates;
					delete d.target;
				}
			}
		};
	}
}());
