(function() {
	'use strict';

	angular.module('ipconfigure.orchid.directive').directive('player', ['$http', '$log', '$timeout', '$rootScope', 'AuthService', 'LowBandwidthStreamService', 'PlayerComponentRegistrar', 'RecordingStyleService', player]);

	function player($http, $log, $timeout, $rootScope, AuthService, LowBandwidthStreamService, PlayerComponentRegistrar, RecordingStyleService) {
		return {
			restrict: 'E',
			replace: true,
			scope: {
				playerId: '=',
				stream: '=',
				playFromTime: '@',
				autoPause: '=',
				instance: '=',
				onremove: '&',
				onfullscreen: '&'
			},
			template: ' ' +
			'<div id="player{{playerId}}" class="player-container">' +
			'  <div class="video" digital-ptz="zoomValue" digital-ptz-settings="zoomSettings"></div>' +
			'  <div class="ripping-container">' +
			'    <img src="data:image/gif;base64,R0lGODlhAQABAAD/ACwAAAAAAQABAAACADs%3D">' +
			'  </div>' +
			'  <div class="mask" ng-show="showMask">{{showMask}}</div>' +
			'  <div class="status">' +
			'    <i class="fa fa-asterisk fa-2x fa-spin"></i>' +
			'  </div>' +
			'  <div class="timeline"></div>' +
			'  <div class="linker" ng-show="unlinkAvailable">' +
			'    <i class="fa fa-link right" ng-show="linked" ng-click="toggleLink($event)"></i>' +
			'    <i class="fa fa-unlink right" ng-show="!linked" ng-click="toggleLink($event)"></i>' +
			'  </div>' +
			'  <img class="resize-anchor" src="img/resize.svg">' +
			'  <div class="resize-right"></div>' +
			'  <div class="resize-bottom"></div>' +
			'  <div class="notification">' +
			'    <div></div>' +
			'    <span></span>' +
			'  </div>' +
			'  <div class="toolbar">' +
			'    <div class="mask" ng-class="{ zoomed: zoomValue > 100 }"></div>' +
			'    <div class="title">{{stream.cameraExpanded.name}} | {{stream.name}} <span ng-show="zoomValue > 100">{{zoomValue}}%</span></div>' +
			'    <div class="buttons">' +
			'      <i class="fa fa-arrows-alt" ng-click="toggleFullscreen($event)"></i>' +
			'      <i class="fa fa-times pull-right" ng-click="closePlayer($event)"></i>' +
			'      <div class="pull-right" style="cursor: pointer; width: 22px; margin-right: 10px;" ng-click="toggleVolume($event)"><i class="fa" ng-show="volumeAvailable" ng-class="{ \'fa-volume-up\': volume === 1, \'fa-volume-down\': volume === 0.5, \'fa-volume-off\': volume === 0 }"></i></div>' +
			'      <i class="fa fa-crosshairs pull-right" style="margin-right: 10px;" ng-show="ptzAvailable" ng-click="togglePtz($event)"></i>' +
			'    </div>' +
			'  </div>' +
			'  <div class="ptz" ng-class="{ \'ptz-disabled\': ptzRequestOutstanding }" ng-show="ptzAvailable && ptzEnabled">' +
			'    <table>' +
			'      <tr>' +
			'        <td width="33%"></td>' +
			'        <td width="33%" align="center" valign="top"><i class="fa fa-caret-up" valign="top" ng-click="moveUp($event)" ng-show="ptzTiltAvailable"></i></td>' +
			'        <td width="33%"></td>' +
			'      </tr>' +
			'      <tr>' +
			'        <td align="left">&nbsp;<i class="fa fa-caret-left" ng-click="moveLeft($event)" ng-show="ptzPanAvailable"></i></td>' +
			'        <td></td>' +
			'        <td align="right"><i class="fa fa-caret-right" ng-click="moveRight($event)" ng-show="ptzPanAvailable"></i>&nbsp;</td>' +
			'      </tr>' +
			'      <tr>' +
			'        <td></td>' +
			'        <td align="center" valign="bottom"><i class="fa fa-caret-down" ng-click="moveDown($event)" ng-show="ptzTiltAvailable"></i> </td>' +
			'        <td align="right" valign="bottom" style="padding-right: 0.1em;">' +
			'          <i class="fa fa-search-minus" style="font-size: 0.5em;" ng-click="zoomOut($event)" ng-show="ptzZoomAvailable"></i>' +
			'          <i class="fa fa-search-plus" style="font-size: 0.5em;" ng-click="zoomIn($event)" ng-show="ptzZoomAvailable"></i>' +
			'        </td>' +
			'      </tr>' +
			'    </table>' +
			'  </div>' +
			'</div>',
			link: function(scope, element) {
				var _linker = element[0].querySelector('.linker'),
					_resizeAnchor = element[0].querySelector('.resize-anchor'),
					_toolbar = element[0].querySelector('.toolbar'),
					_timelineContainer = element[0].querySelector('.timeline'),
					_ptzContainer = element[0].querySelector('.ptz'),
					_notificationContainer = element[0].querySelector('.notification'),
					_notificationSpan = element[0].querySelector('.notification span'),
					_resizer = {
						bottom: element[0].querySelector('.resize-bottom'),
						right: element[0].querySelector('.resize-right')
					},
					_rippingContainer = element[0].querySelector('.ripping-container'),
					_rippingBackgroundImage = _rippingContainer.firstElementChild,
					_closeButton = element[0].querySelector('.pull-right'),
					_statusOverlay = element[0].querySelector('.status'),
					_video,
					_timeline,
					_hideOverlaysTimer,
					_isFullScreen = false,
					_ripping = {
						active: false,
						loading: false,
						loadedTime: 0,
						desiredTime: 0
					},
					_renegotiateTimer,
					_lbMetadata = {
						resource: undefined,
						lastStart: 0
					};

// TODO persist zoomSettings when we have a playerId (instead of just a streamId)
//				scope.$watch('zoomSettings', function (newValue) {
//					var stageMatrix = JSON.parse(localStorage.stageMatrix),
//						updated;
//
//					updated = stageMatrix.some(function (row) {
//						return row.some(function (cell) {
//							if (cell.streamId === scope.stream.id) {
//								cell.zoomSettings = newValue;
//								return true;
//							}
//						});
//					});
//
//					if (updated === true) {
//						localStorage.stageMatrix = JSON.stringify(stageMatrix);
//					}
//				});

				scope.closePlayer = function($event) {
					if ($event.defaultPrevented === false) {
						if (_isFullScreen === true) {
							scope.toggleFullscreen($event);
						}

						scope.onremove();
					}
				};

				scope.toggleFullscreen = function($event) {
					var display;

					if ($event.defaultPrevented === false) {
						_isFullScreen = !_isFullScreen;

						scope.toggleFullScreen();
						scope.onfullscreen();

						display = (_isFullScreen === true) ? 'none' : '';

						_linker.style.display = display;
						_resizeAnchor.style.display = display;
						_resizer.bottom.style.display = display;
						_resizer.right.style.display = display;
					}
				};

				scope.toggleLink = function($event) {
					if ($event.defaultPrevented === false) {
						scope.toggleLink();
					}
				};

				scope.togglePtz = function($event) {
					if ($event.defaultPrevented === false) {
						// show/hide ptz controls
						scope.ptzEnabled = !scope.ptzEnabled;
					}
				};

				scope.toggleVolume = function($event) {
					if ($event.defaultPrevented === false) {
						switch (scope.volume) {
							case 0:
								scope.volume = 0.5;
								break;
							case 0.5:
								scope.volume = 1;
								break;
							case 1:
								scope.volume = 0;
								break;
						}

						_video.setVolume(scope.volume);
					}
				};

				function _showOverlays() {
					_toolbar.style.top = '0px';

					if (scope.linked === true) {
						_linker.style.bottom = '3px';
					}

					_timelineContainer.style.bottom = '0px';
					_resizer.bottom.style.opacity = 0.75;
					_resizer.right.style.opacity = 0.75;
					_resizeAnchor.style.opacity = 0.75;

					if (scope.notificationVisible === true) {
						_notificationContainer.style.top = '28px';
					}
				}

				function _hideOverlays() {
					_toolbar.style.top = '-' + _toolbar.clientHeight + 'px';

					if (scope.linked === true) {
						_linker.style.bottom = ('-' + _linker.clientHeight + 'px');
						_timelineContainer.style.bottom = '-' + _timelineContainer.clientHeight;
					}

					_resizer.bottom.style.opacity = '';
					_resizer.right.style.opacity = '';
					_resizeAnchor.style.opacity = '';

					if (scope.notificationVisible === true) {
						_notificationContainer.style.top = '0px';
					}
				}

				function _requestPlaybackStream(time, callback) {
					var playbackUrl = scope.instance.rtspProtocol + '://' + ipConfigure.core.parseUri(scope.instance.baseUrl).host + ':' + ((scope.instance.rtspPort ? scope.instance.rtspPort : 40000)) + '/orchid/playback?id=' + scope.stream.id + '&start=' + time + '&rate=1&token=' + Math.floor(Math.random() * 100000);

					$log.debug('_requestPlaybackStream', playbackUrl);

					// provide playback stream (url is determined by convention now, bypassing the web service)
					callback(playbackUrl);
				}

				function _requestBaseUrl(options, callback) {
					// treat the server real-nice-like by immediately destroying any cached stream we're holding
					if (angular.isUndefined(_lbMetadata.resource) === false) {
						// tell the video component to stop pulling the frame
						callback('', 1);
						_destroyLowBandwidthStream();
					}

					_lbMetadata.lastStart = options.start;

					// ask low-bandwidth service for a frame puller thing
					LowBandwidthStreamService.createStream(scope.stream, options, function(success, resource) {
						if (success === true) {
							// hold onto resource so we can DELETE
							_lbMetadata.resource = resource;

							if (resource.state.status !== 'pending') {
								callback(resource.href + '/frame', resource.fps);
							}
						}
						else {
							_lbMetadata.resource = resource;
							_destroyLowBandwidthStream();

							callback(false, resource.state.description);
						}
					});
				}

				function _move(x, y, z) {
					if (scope.ptzRequestOutstanding === false) {
						scope.ptzRequestOutstanding = true;

						$http.post(scope.instance.baseUrl + '/service/cameras/' + scope.stream.camera.id + '/position', { x: x, y: y, z: z })
							.success(function() {
								scope.ptzRequestOutstanding = false;
							})
							.error(function(data, status, headers, config) {
								alert('PTZ command failed:\n\n' + data.reason);
								$log.error(data, status, headers, config);

								scope.ptzRequestOutstanding = false;
							});
					}
				}

				function _renegotiate() {
					if (_video.isLive) {
						_video.playLive();
					}
					else {
						_video.playFromTime(scope.linked === false ? _timeline.getTime() : PlayerComponentRegistrar.getTime());
					}
				}

				function _ripFrame() {
					var sizedParameter = '',
						orientation = '';

					$log.debug('_ripFrame', 'waiting', true, 'active', _ripping.active);

					if (angular.isUndefined(scope.stream.aspectRatio) === false) {
						if (scope.stream.aspectRatio[0] / scope.stream.aspectRatio[1] > _rippingContainer.clientWidth / _rippingContainer.clientHeight) {
							sizedParameter = '&width=' + Math.round(_rippingContainer.clientWidth / 3);
							orientation = 'landscape';
						}
						else {
							sizedParameter = '&height=' + Math.round(_rippingContainer.clientHeight / 3);
							orientation = 'portrait';
						}
					}

					if (_ripping.active === true && _ripping.desiredTime !== _ripping.loadedTime) {
						_ripping.loading = true;
						_ripping.loadedTime = _ripping.desiredTime;
						_rippingBackgroundImage.className = orientation;
						_rippingBackgroundImage.src = scope.stream.href + '/frame?fallback=true&time=' + (_ripping.desiredTime - PlayerComponentRegistrar.getTimeDifference()) + sizedParameter;
					}
					else {
						_ripping.loading = false;
					}
				}

				function _setPtzFontSize() {
					// adjust the ptz controls size
					_ptzContainer.style.fontSize = ((element[0].offsetWidth - 120) / 100) + 'em';
				}

				function _destroyLowBandwidthStream() {
					if (_lbMetadata.resource) {
						LowBandwidthStreamService.destroyStream(_lbMetadata.resource);
						_lbMetadata.resource = undefined;
					}
				}

				function _displayNotification(message) {
					// display a message
					if (message) {
						scope.notificationVisible = true;
						_notificationContainer.style.top = (scope.overlaysVisible) ? '28px' : '0px';

						if (message === 'Unable to display video at this time' && RecordingStyleService.determineRecordingStyleFromStream(scope.stream).name === 'Continuous recording without motion detection') {
							message = 'Live view unavailable without motion detection';
						}
						else if (message === 'Unable to display video at this time' && RecordingStyleService.determineRecordingStyleFromStream(scope.stream).name === 'Proxy only, no video will be recorded') {
							message = $rootScope.compatibilityMode ? 'Low bandwidth unavailable in proxy-only mode' : 'Playback unavailable in proxy-only mode';
						}

						_notificationSpan.textContent = message;
					}
					else {
						scope.notificationVisible = false;
						_notificationContainer.style.top = '-28px';
					}
				}

				scope.overlaysVisible = false;
				scope.notificationVisible = false;
				scope.linked = true;
				scope.fullScreen = false;
				scope.ptzAvailable = false;
				scope.ptzEnabled = false;
				scope.ptzRequestOutstanding = false;
				scope.volume = 0;
				scope.volumeAvailable = false;
				scope.backgroundLive = false;

				scope.zoomIn = function($event) {
					if ($event.defaultPrevented === false) {
						_move(0, 0, 0.1);
					}
				};
				scope.zoomOut = function($event) {
					if ($event.defaultPrevented === false) {
						_move(0, 0, -0.1);
					}
				};
				scope.moveUp = function($event) {
					if ($event.defaultPrevented === false) {
						_move(0, 0.1, 0);
					}
				};
				scope.moveRight = function($event) {
					if ($event.defaultPrevented === false) {
						_move(0.1, 0, 0);
					}
				};
				scope.moveDown = function($event) {
					if ($event.defaultPrevented === false) {
						_move(0, -0.1, 0);
					}
				};
				scope.moveLeft = function($event) {
					if ($event.defaultPrevented === false) {
						_move(-0.1, 0, 0);
					}
				};

				scope.rip = function(time) {
					// user is ripping, no need to continue playing video
					if (_video.isPlaying === true) {
						_video.stop();
					}

					if (time === false) {
						// simply track that the user is no longer ripping -- let state changes hide the rippingContainer
						_ripping.active = false;
					}
					else {
						// simply track that the user is ripping -- let the image load handler show the rippingContainer
						_ripping.active = true;
						_ripping.desiredTime = time;

						if (_ripping.loading === false) {
							_ripFrame();
						}
					}
				};

				scope.toggleLink = function() {
					if (scope.linked === true) {
						// unlink the video from the master timeline
						if (scope.fullScreen === true) {
							alert('You cannot unlink while fullscreen.');
							return;
						}

						_timeline.trackLive();

						$rootScope.$broadcast('unlink', scope.playerId);

						_linker.style.bottom = '-10px';
						_linker.style.color = 'black';

						_timelineContainer.style.left = '0px';
					}
					else {
						// link the video to the master timeline
						$rootScope.$broadcast('relink', scope.playerId);

						_linker.style.bottom = '3px';
						_linker.style.color = '';

						_timelineContainer.style.left = '100%';
					}

					scope.linked = !scope.linked;
					_video.linked = scope.linked;
				};

				scope.toggleFullScreen = function() {
					scope.fullScreen = !scope.fullScreen;
					$rootScope.$broadcast('fullScreen', scope.playerId);
				};

				// adjust positions/sizing with the stage resizes
				scope.$on('resize.stage', function() {
					_setPtzFontSize();

					// force timeline to resize
					_timeline.resize();
				});

				// clean up the directive on $destroy
				scope.$on('$destroy', function() {
					PlayerComponentRegistrar.unregisterComponent(scope.playerId);

					_video.destroy();
					_timeline.destroy();

					$timeout.cancel(_renegotiateTimer);

					_destroyLowBandwidthStream();
				});

				function _init() {
					// because of FBGST we can't just simply use mouseenter/mouseleave, so we throw in mousemove
					element.bind('mousemove', function() {
						if (scope.overlaysVisible === false) {
							scope.overlaysVisible = true;
							_showOverlays();
						}

						if (_hideOverlaysTimer) {
							clearTimeout(_hideOverlaysTimer);
						}

						_hideOverlaysTimer = setTimeout(function() {
							_hideOverlays();
							scope.overlaysVisible = false;
						}, 2500);
					});

					_video = new ($rootScope.compatibilityMode ? ipConfigure.components.videoCompatibility : ipConfigure.components.video)('player' + scope.playerId + ' .video', {
						authentication: {
							username: 'sessionId',
							password: AuthService.getAuthenticatedUser().sessionId
						},
						backgroundColor: '#000000',
						debug: false,
						getTime: function() {
							return scope.linked === false ? _timeline.getTime() : PlayerComponentRegistrar.getTime();
						},
						requestPlaybackStream: _requestPlaybackStream,
						requestBaseUrl: _requestBaseUrl,
						onAudioKnown: function(hasAudio) {
							scope.volumeAvailable = hasAudio;
						},
						onEndReached: function() {
							$timeout.cancel(_renegotiateTimer);
							_renegotiateTimer = $timeout(_renegotiate, 10000);
						},
						onStateChange: function(state, message) {
							if (state === 'void' || state === 'null' || state === 'ready') {
								scope.showMask = false;
								_statusOverlay.style.display = '';
							}
							else if (state === 'loading') {
								scope.showMask = true;
								_statusOverlay.style.display = 'block';

								$timeout.cancel(_renegotiateTimer);
							}
							else if (state === 'playing') {
								scope.showMask = false;
								_statusOverlay.style.display = '';

								// hide the rip container now that we're playing
								_rippingContainer.style.display = '';
							}
							else if (state === 'error') {
								scope.showMask = true;
								_statusOverlay.style.display = '';

								$timeout.cancel(_renegotiateTimer);
								_renegotiateTimer = $timeout(_renegotiate, 10000);
							}

							_displayNotification(message);
						},
						onPlayFromTime: function() {
							scope.ptzPanAvailable = false;
							scope.ptzTiltAvailable = false;
							scope.ptzZoomAvailable = false;
							scope.ptzAvailable = false;
						},
						onPlayLive: function() {
							if ((AuthService.getAuthenticatedUser().role === 'Manager' || AuthService.getAuthenticatedUser().role === 'Administrator') && angular.isDefined(scope.stream.cameraExpanded.capabilities.camera.PTZ) === true) {
								scope.ptzPanAvailable = (scope.stream.cameraExpanded.capabilities.camera.PTZ.pan === 'true');
								scope.ptzTiltAvailable = (scope.stream.cameraExpanded.capabilities.camera.PTZ.tilt === 'true');
								scope.ptzZoomAvailable = (scope.stream.cameraExpanded.capabilities.camera.PTZ.zoom === 'true');
								scope.ptzAvailable = (scope.ptzPanAvailable || scope.ptzTiltAvailable || scope.ptzZoomAvailable);
							}
						},
						stream: scope.stream
					});

					_timeline = new ipConfigure.components.timeline('player' + scope.playerId + ' .timeline', {
						debug: false,
						draggableIndicator: false,
						onTimeChange: function(time, source) {
							var diff = Date.now() - time;

							scope.rip(false, { source: source });

							if (diff < 5000) {
								if (_video.isLive === false) {
									_timeline.setFineTime(Date.now());
									_video.playLive();
								}
							}
							else {
								_timeline.setFineTime(time);
								_video.playFromTime(time);
							}
						},
						onRip: function(time, metadata) {
							if (_video.isPlaying === true) {
								_video.stop();
							}

							scope.rip(time, metadata);
						}
					});

					if (scope.playFromTime) {
						_video.playFromTime(parseInt(scope.playFromTime, 10), scope.autoPause);
					}
					else {
						_video.playLive();
					}

					scope.unlinkAvailable = (AuthService.getAuthenticatedUser().role !== 'Live Viewer');

					PlayerComponentRegistrar.registerComponent(scope, _timeline, _video);

					_rippingBackgroundImage.addEventListener('error', function() {
						var wait = (scope.linked === false || PlayerComponentRegistrar.getComponents().length < 5) ? 333 : 3333;

						$timeout(function() {
							_ripFrame();
						}, wait);
					});

					_rippingBackgroundImage.addEventListener('load', function() {
						var wait = (scope.linked === false || PlayerComponentRegistrar.getComponents().length < 5) ? 333 : 3333;

						if (_ripping.active === true) {
							// if the user is still ripping and an image has loaded, ensure that the container is displayed
							_rippingContainer.style.display = 'block';
						}

						$timeout(function() {
							_ripFrame();
						}, wait);
					});

					_closeButton.addEventListener('mousedown', function(mouseDownEvent) {
						// this is in attempt to prevent a bug that happens randomly where d3
						// considers a click a drag, and cancels the click... so this is here
						// to stop the user from dragging the close button
						mouseDownEvent.preventDefault();
						mouseDownEvent.stopPropagation();
					});

					_setPtzFontSize();
				}

				// initialize the components after $apply() is done setting this directive up
				$timeout(_init, 0);
			}
		};
	}
}());
