(function() {
	'use strict';

	angular.module('ipconfigure.orchid.filter').filter('timeDuration', [timeDurationFilter]);

	function timeDurationFilter() {
		// hh:mm:ss
		function duration(diff) {
			var absDiff = Math.abs(diff),
				s = Math.floor(absDiff / 1000 % 60),
				m = Math.floor(absDiff / 60000 % 60),
				h = Math.floor(absDiff / 3600000 % 24) + Math.floor(absDiff / 86400000) * 24;

			return (h + ':' + ('0' + m).slice(-2) + ':' + ('0' + s).slice(-2)).replace(/^00:/, '').replace(/.000$/, '');
		}

		function pluralize(value, key) {
			return value + ' ' + ((value !== 1) ? key + 's' : key);
		}

		// humanized (2 days, 3 hours, and 3 seconds)
		function durationHumanized(diff, maxComponents) {
			var total = Math.abs(diff),
				timeElement,
				parts = [],
				valueTotal = 1000 * 60 * 60 * 24,
				values = [
					{ key: 'day', value: 24, msValue: 86400000 },
					{ key: 'hour', value: 60, msValue: 3600000 },
					{ key: 'minute', value: 60, msValue: 60000 },
					{ key: 'second', value: 1000, msValue: 1000 }];

			while (values.length > 0 && !(typeof maxComponents === 'number' && maxComponents > 0 && parts.length >= maxComponents)) {
				timeElement = values.shift();

				if (total >= valueTotal) {
					// track this time element
					parts.push(pluralize(Math.floor(total / valueTotal), timeElement.key));

					// adjust the total
					total -= Math.floor(total / valueTotal) * timeElement.msValue;
				}

				// adjust the total
				valueTotal = valueTotal / timeElement.value;
			}

			// default
			if (parts.length === 0) {
				return '0 seconds';
			}
			else {
				return parts.join(', ').replace(/(,)([^,]*)$/, ', and$2');
			}
		}

		return function(milliseconds, humanize, maxComponents) {
			var normalizedMilliseconds = parseInt(milliseconds, 10);

			// fix bad input
			if (isNaN(normalizedMilliseconds) === true || normalizedMilliseconds < 0) {
				normalizedMilliseconds = 0;
			}

			if (humanize === true) {
				return durationHumanized(normalizedMilliseconds, maxComponents);
			}
			else {
				return duration(normalizedMilliseconds);
			}
		};
	}
}());
