window.ipConfigure = window.ipConfigure || {};
ipConfigure.core = ipConfigure.core || {};

(function() {
	'use strict';

	ipConfigure.core.mergeOptions = function(obj1, obj2) {
		var attrName,
			obj3 = {};

		for (attrName in obj1) {
			if (obj1.hasOwnProperty(attrName) === true) {
				obj3[attrName] = obj1[attrName];
			}
		}

		for (attrName in obj2) {
			if (obj2.hasOwnProperty(attrName) === true) {
				obj3[attrName] = obj2[attrName];
			}
		}

		return obj3;
	};

	ipConfigure.core.compareVersions = function(installed, required) {
		function makeComparable(version) {
			var comparableVersion = '',
				pieces = version.split('.');

			pieces.forEach(function(piece) {
				while (piece.length < 5) {
					piece = '0' + piece;
				}

				comparableVersion = comparableVersion + piece;
			});

			return comparableVersion;
		}

		return typeof installed !== 'undefined' && makeComparable(required) <= makeComparable(installed);
	};

	ipConfigure.core.isEmpty = function(obj) {
		// undefined, null, and empty string are "empty"
		if (typeof obj === 'undefined' || obj === null || obj === '') {
			return true;
		}

		// Assume if it has a length property with a non-zero value that that property is correct.
		if (obj.length > 0) {
			return false;
		}

		if (obj.length === 0) {
			return true;
		}

		// Otherwise, does it have any properties of its own?
		// Note that this doesn't handle
		// toString and toValue enumeration bugs in IE < 9
		for (var key in obj) {
			if (obj.hasOwnProperty(key)) {
				return false;
			}
		}

		return true;
	};

	ipConfigure.core.splitCamelCase = function(s) {
		return s.replace(/([A-Z]+)/g, ' $1').replace(/([A-Z][a-z])/g, ' $1');
	};

	ipConfigure.core.roundDateToHours = function(date, interval) {
		var newDate = new Date(date.getTime()),
			h = newDate.getHours() + newDate.getMinutes() / 60 + newDate.getSeconds() / 3600 + date.getMilliseconds() / 3600000;

		newDate.setMinutes(0);
		newDate.setSeconds(0);
		newDate.setMilliseconds(0);
		newDate.setHours(Math.round(h / interval) * interval);

		return newDate;
	};

	ipConfigure.core.roundDateToMinutes = function(date, interval) {
		var newDate = new Date(date.getTime()),
			m = newDate.getMinutes() + newDate.getSeconds() / 60 + date.getMilliseconds() / 60000;

		newDate.setSeconds(0);
		newDate.setMilliseconds(0);
		newDate.setMinutes(Math.round(m / interval) * interval);

		return newDate;
	};

	ipConfigure.core.floorDateToMinutes = function(date, interval) {
		var newDate = new Date(date.getTime()),
			m = newDate.getMinutes() + newDate.getSeconds() / 60 + date.getMilliseconds() / 60000;

		newDate.setSeconds(0);
		newDate.setMilliseconds(0);
		newDate.setMinutes(Math.floor(m / interval) * interval);

		return newDate;
	};

	ipConfigure.core.ceilDateToMinutes = function(date, interval) {
		var newDate = new Date(date.getTime()),
			m = newDate.getMinutes() + newDate.getSeconds() / 60 + date.getMilliseconds() / 60000;

		newDate.setSeconds(0);
		newDate.setMilliseconds(0);
		newDate.setMinutes(Math.ceil(m / interval) * interval);

		return newDate;
	};

	ipConfigure.core.fillRectangle = function(containerLeft, containerTop, containerWidth, containerHeight, objectWidth, objectHeight) {
		var rectangle = {
			left: 0,
			top: 0,
			width: 0,
			height: 0
		};

		if (containerWidth / containerHeight > objectWidth / objectHeight) {
			rectangle.width = objectWidth * containerHeight / objectHeight;
			rectangle.height = containerHeight;
			rectangle.top = containerTop;
			rectangle.left = containerLeft + (containerWidth - rectangle.width) / 2;
		}
		else {
			rectangle.width = containerWidth;
			rectangle.height = objectHeight * containerWidth / objectWidth;
			rectangle.top = containerTop + (containerHeight - rectangle.height) / 2;
			rectangle.left = containerLeft;
		}

		return rectangle;
	};

	// parseUri 1.2.2
	// (c) Steven Levithan <stevenlevithan.com>
	// MIT License
	ipConfigure.core.parseUri = function(str) {
		var o = ipConfigure.core.parseUri.options,
			m = o.parser[o.strictMode ? 'strict' : 'loose'].exec(str),
			uri = {},
			i = 14;

		while (i--) {
			uri[o.key[i]] = m[i] || '';
		}

		uri[o.q.name] = {};
		uri[o.key[12]].replace(o.q.parser, function($0, $1, $2) {
			if ($1) {
				uri[o.q.name][$1] = $2;
			}
		});

		return uri;
	};

	ipConfigure.core.parseUri.options = {
		strictMode: false,
		key: ['source', 'protocol', 'authority', 'userInfo', 'user', 'password', 'host', 'port', 'relative', 'path', 'directory', 'file', 'query', 'anchor'],
		q: {
			name: 'queryKey',
			parser: /(?:^|&)([^&=]*)=?([^&]*)/g
		},
		parser: {
			strict: /^(?:([^:\/?#]+):)?(?:\/\/((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?))?((((?:[^?#\/]*\/)*)([^?#]*))(?:\?([^#]*))?(?:#(.*))?)/,
			loose: /^(?:(?![^:@]+:[^:@\/]*@)([^:\/?#.]+):)?(?:\/\/)?((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(?:#(.*))?)/
		}
	};

	function getDevice() {
		var a = navigator.userAgent || navigator.vendor || window.opera,
			isMobile,
			isTablet,
			operatingSystem = 'Unknown';

		if (navigator.appVersion.indexOf('Win') !== -1) {
			operatingSystem = 'Windows';
		}
		else if (navigator.appVersion.indexOf('Mac') !== -1) {
			operatingSystem = 'MacOS';
		}
		else if (navigator.appVersion.indexOf('Android') !== -1) {
			operatingSystem = 'Android';
		}
		else if (navigator.appVersion.indexOf('Linux') !== -1) {
			operatingSystem = 'Linux';
		}

		// via http://detectmobilebrowsers.com/
		isMobile = (/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows ce|xda|xiino/i.test(a) ||
		/1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/i.test(a.substr(0, 4)));

		// via http://detectmobilebrowsers.com/about
		isTablet = /android|ipad|playbook|silk/i.test(a);

		return {
			mobile: isMobile,
			tablet: isTablet,
			desktop: !isMobile && !isTablet,
			operatingSystem: operatingSystem
		};
	}

	ipConfigure.core.getBrowser = function() {
		var ua = navigator.userAgent,
			M = ua.match(/(opera|chrome|safari|firefox|msie|trident(?=\/))\/?\s*(\d+)/i) || [],
			tem,
			device = getDevice(),
			result = {
				name: '',
				version: '',
				platform: navigator.platform,
				isDesktop: device.desktop,
				isMobile: device.mobile,
				isTablet: device.tablet,
				operatingSystem: device.operatingSystem
			};

		if (/trident/i.test(M[0])) {
			tem = /\brv[ :]+(\d+)/g.exec(ua) || [];
			result.name = 'MSIE';
			result.version = (parseInt(tem[1], 10) || 0);
		}
		else if (M[1] === 'Chrome') {
			tem = ua.match(/\bOPR\/(\d+)/);
			if (tem !== null) {
				result.name = 'Opera';
				result.version = parseInt(tem[1], 10);
			}
			else {
				result.name = M[1];
				result.version = parseInt(M[2], 10);
			}
		}
		else {
			if (M.length === 0 && navigator.platform === 'iPad' && navigator.appName === 'Netscape') {
				M = ['Safari/7.0', 'Safari', 7];
			}

			M = M[2] ? [M[1], M[2]] : [navigator.appName, navigator.appVersion, '-?'];

			if ((tem = ua.match(/version\/(\d+)/i)) !== null) {
				M.splice(1, 1, tem[1]);
			}

			result.name = M[0];
			result.version = parseInt(M[1], 10);
		}

		return result;
	};

	ipConfigure.core.screenLog = function(s) {
		var lines = 5,
			log = document.querySelector('#log'),
			logParts = log.innerHTML.split('<br>');

		if (logParts.length === 1 && logParts[0] === '') {
			logParts = [];
		}

		logParts.push(s);

		while (logParts.length > lines) {
			logParts.splice(0, 1);
		}

		log.innerHTML = logParts.join('<br>');
	};

	// for querySelectorAll - via http://toddmotto.com/ditch-the-array-foreach-call-nodelist-hack/
	ipConfigure.core.forEach = function(array, callback, scope) {
		for (var i = 0; i < array.length; i++) {
			callback.call(scope, array[i], i);
		}
	};

	ipConfigure.core.some = function(array, callback, scope) {
		// TODO make this work more like native some
		for (var i = 0; i < array.length; i++) {
			if (callback.call(scope, array[i], i) === false) {
				break;
			}
		}
	};
}());
