(function() {
	'use strict';

	angular.module('ipconfigure.orchid').factory('HttpResponseErrorHandler', ['$q', '$rootScope', '$timeout', HttpResponseErrorHandler]);

	function HttpResponseErrorHandler($q, $rootScope, $timeout) {
		var orchidDestroyed = false;

		return {
			response: function(response) {
				if (response.config.url === '/ping') {
					// if the server came back while we're waiting for it, go ahead and refresh
					if (orchidDestroyed === true) {
						window.location.reload();
					}
				}

				return response;
			},
			responseError: function(rejection) {
				// handle 403 forbidden by forcing a logged in user out
				if (rejection.status === 403) {
					// force the user out
					if (window.location.href.indexOf('configuration.html') > -1) {
						// if they're on the configuration page, just bump them back to the root
						window.location.href = '/';
					}
					else {
						$rootScope.$broadcast('logOut');
					}

					// adjust the status to zero so that all callbacks ignore it
					rejection.status = 0;
				}

				// if requests to '/ping' ever fail with status of 0, it means the orchid web server is non-responsive
				else if (rejection.status === -1 && rejection.config.method === 'GET' && (rejection.config.url.lastIndexOf('/ping') + 5) === rejection.config.url.length && orchidDestroyed === false) {
					orchidDestroyed = true;

					// we're doing this as opposed to using the angular stack, because if we try to use the ngInclude and the server isn't available, it won't be able to load
					// the view html file anyway...
					$timeout(function() {
						$rootScope.$broadcast('$destroy');

						// hide the header buttons
						[].forEach.call(document.querySelectorAll('#header i.fa'), function(el) {
							el.style.visibility = 'hidden';
						});

						// destroy the admin
						document.getElementById('admin').parentNode.removeChild(document.getElementById('admin'));

						// show an error message
						document.getElementById('main').className = '';
						document.getElementById('main').innerHTML = ' ' +
							'<div><div><div class="row" style="padding-top: 36px;"><div class="col-md-6 col-md-offset-3"><div class="well well-lg">' +
							'   <h1>Orchid Service Unresponsive</h1>' +
							'   <p>It seems that the Orchid server is offline or otherwise unreachable.  The server should restart itself while the ambulance is en route.</p>' +
							'   <p>If this error persists or you do not know how to proceed, feel free to give our support line a call @ +1-877-207-1112 (Option 2).</p>' +
							'   <div style="overflow: hidden; border-bottom: 1px solid gray; width: 100%; height: 66px; position: relative;">' +
							'       <i class="fa fa-ambulance fa-5x"></i>' +
							'   </div>' +
							'</div></div></div></div></div>';
					}, 30000);
				}

				return $q.reject(rejection);
			}
		};
	}
}());
