(function() {
	'use strict';

	angular.module('ipconfigure.orchid.service').factory('RecordingStyleService', [RecordingStyleService]);

	function RecordingStyleService() {
		var defaultRecordingStyleIndex = 0,
			recordingStyleTable = [
			{
				group: 'Continuous recording options',
				name: 'Continuous recording with motion detection',
				test: function(stream) {
					return (stream.recordingConfiguration.mode === 'ALL_FEATURES_MODE' && stream.recordingConfiguration.filesplit.motionMode === '0' && stream.recordingConfiguration.motionReducer === 'false');
				},
				updateConfiguration: function(stream) {
					stream.recordingConfiguration.filesplit.motionMode = '0';
					stream.recordingConfiguration.filesplit.recordState = 'true';
					stream.recordingConfiguration.mode = 'ALL_FEATURES_MODE';
					stream.recordingConfiguration.motionReducer = 'false';
				}
			},
			{
				group: 'Continuous recording options',
				name: 'Continuous recording without motion detection',
				test: function(stream) {
					return (stream.recordingConfiguration.mode === 'EMBEDDED_LITE_MODE');
				},
				updateConfiguration: function(stream) {
					stream.recordingConfiguration.filesplit.motionMode = '0';
					stream.recordingConfiguration.filesplit.recordState = 'true';
					stream.recordingConfiguration.mode = 'EMBEDDED_LITE_MODE';
					stream.recordingConfiguration.motionReducer = 'false';
				}
			},
			{
				group: 'Continuous recording options',
				name: 'Continuous recording with motion adaptive frame rate',
				test: function(stream) {
					return (stream.recordingConfiguration.mode === 'ALL_FEATURES_MODE' && stream.recordingConfiguration.filesplit.motionMode === '0' && stream.recordingConfiguration.motionReducer === 'true');
				},
				updateConfiguration: function(stream) {
					stream.recordingConfiguration.filesplit.motionMode = '0';
					stream.recordingConfiguration.filesplit.recordState = 'true';
					stream.recordingConfiguration.mode = 'ALL_FEATURES_MODE';
					stream.recordingConfiguration.motionReducer = 'true';
				}
			},
			{
				group: 'Non-recording options',
				name: 'Proxy only, no video will be recorded',
				test: function(stream) {
					return (stream.recordingConfiguration.mode === 'PROXY_ONLY_MODE');
				},
				updateConfiguration: function(stream) {
					stream.recordingConfiguration.filesplit.motionMode = '0';
					stream.recordingConfiguration.filesplit.recordState = 'false';
					stream.recordingConfiguration.mode = 'PROXY_ONLY_MODE';
					stream.recordingConfiguration.motionReducer = 'false';
				}
			}//,
			//{
			//	group: 'Noncontinuous recording',
			//	name: 'Recording only when motion detected',
			//	test: function(stream) {
			//		return (stream.recordingConfiguration.mode === 'ALL_FEATURES_MODE' && stream.recordingConfiguration.filesplit.motionMode === '2');
			//	},
			//	updateConfiguration: function(stream) {
			//		stream.recordingConfiguration.filesplit.motionMode = '2';
			//		stream.recordingConfiguration.filesplit.recordState = 'false';
			//		stream.recordingConfiguration.mode = 'ALL_FEATURES_MODE';
			//		stream.recordingConfiguration.motionReducer = 'false';
			//	}
			//}
		];

		return {
			getAllRecordingStyles: function() {
				return recordingStyleTable;
			},
			getDefaultRecordingStyle: function() {
				return recordingStyleTable[defaultRecordingStyleIndex];
			},
			determineRecordingStyleFromStream: function(stream) {
				var determinedStyle;

				determinedStyle = recordingStyleTable.filter(function(style) {
					return style.test(stream);
				});

				if (determinedStyle.length === 1) {
					return determinedStyle[0];
				}
				else {
					window.alert('Unable to determine recording style');

					// just return the default
					return recordingStyleTable[defaultRecordingStyleIndex];
				}
			},
			setRecordingConfigurationForStreamFromRecordingStyle: function(stream) {
				var determinedStyle;

				determinedStyle = recordingStyleTable.some(function(style) {
					var match = stream.recordingStyle === style.name;

					if (match === true) {
						style.updateConfiguration(stream);
					}

					return match;
				});

				return determinedStyle;
			}
		};
	}
}());
