(function() {
	'use strict';

	angular.module('ipconfigure.orchid.service').factory('TimeDeltaService', ['$http', '$log', '$q', 'BaseServiceUrl', TimeDeltaService]);

	function TimeDeltaService($http, $log, $q, BaseServiceUrl) {
		var differenceInfoPromise = $q.defer(),
			got = false;

		// using window.clientServerTime as our data source so that ipc-components (outside of angular) can use the values as well
		window.clientServerTime = {
			initialized: false,
			failed: false,
			client: 0,
			server: 0,
			diff: 0
		};

		function fetchTime() {
			if (got === false) {
				got = true;

				// fetch time from the base instance and satisfy promises
				$http.get(BaseServiceUrl + '/service/time')
					.success(function(response) {
						var client = Date.now(),
							server = parseInt(response, 10);

						window.clientServerTime = {
							initialized: true,
							failed: false,
							client: client,
							server: server,
							diff: client - server
						};

						differenceInfoPromise.resolve(window.clientServerTime);
					})
					.error(function(data, status, headers, config) {
						// log real errors
						$log.error('TimeDeltaService::fetchTime error', data, status, headers, config);

						// fake some data for now in attempt to let Orchid run
						window.clientServerTime = {
							initialized: true,
							failed: true,
							client: Date.now(),
							diff: 0
						};

						differenceInfoPromise.resolve(window.clientServerTime);
					});
			}
		}

		// NOTE: we delay fetchTime (instead of calling it here in the constructor, because
		// the time service requires authentication which the user will not happen at this point.
		// I suppose more ideally, this would be some sign-in dependency that gets called along
		// with others before the app can truly load and be seen.

		return {
			getDifference: function() {
				fetchTime();

				return window.clientServerTime.diff;
			},
			fetchDifferenceInfo: function() {
				fetchTime();

				return differenceInfoPromise.promise;
			}
		};
	}
}());
